clear
clc
global D 
D = 0.076;


getNotchFunctions;
getFullFunctions;
% % % Input parameters
% Geometry


% Crack Growth
C = 5.3*10^-12;
m = 3;
Kth = 0;
% Loading
S = 100;



% ID indicates which configuartions will be added to the plot. 
ID = [7,8,9,10];
% % Style defintiions
% line_styles = {'-b','-r','-k','--b','--r','--k','-.b','-.r','-.'}; % Only
% Lines
% line_styles = {'bo','r+','ks','bx','rp','k^','b>','r<','h'}; % Only dots

line_styles = {'-bo','-r+','-ks','--bx','--rp','--k^','-.b>','-.r<','-.h'};

legend_names = {};
% % Looping function initializer
Cycles_Step = [];
Cycles_Cum = 0;
Cycles_Max = [];


% % % Crack
num_steps = 20; % How many intervals will be plotted
ai = 0.0002; % Initial crack depth
af = 0.038; % Final crack depth
step = (af-ai)/num_steps; 
crack_array = ai:step:af;

count = 1;
for j =1:length(ID)
    step = (af-ai)/num_steps; 
    if ID(j) == 10 && ai./D<0.02
        ai_tmp = ai+10^-3;
        crack_array = ai_tmp:step:af;
    else
        crack_array = ai:step:af;
    end
    
    if ID(j) == 1
        coreFun = T_side;
        legend_names{count} = 'Tension, Side';
        count = count + 1;
    elseif ID(j) == 2
        coreFun = T_bot;
        legend_names{count} = 'Tension, Bot';
        count = count + 1;
    elseif ID(j) == 3
        coreFun = T_bot;
        legend_names{count} = 'Tension, Enclosed';
        count = count + 1;  
    elseif ID(j) == 4
        coreFun = B_side;
        legend_names{count} = 'Bending, Side';
        count = count + 1;
    elseif ID(j) == 5
        coreFun = B_bot;
        legend_names{count} = 'Bending, Bot';
        count = count + 1;        
    elseif ID(j) == 6
        coreFun = @(x) a.*exp(-b.*(x./D))+c+(x./D).*d;
        legend_names{count} = 'Bending, Enclosed';
        count = count + 1;    
    elseif ID(j) == 7
        coreFun = ES_Side;
        legend_names{count} = 'Start side, grow ellip, H2';
        count = count + 1;   
    elseif ID(j) == 8
        coreFun = CS_Side;
        legend_names{count} = 'Start side, grow circ, H2M';
        count = count + 1;   
    elseif ID(j) == 9
        coreFun = CS_Bot;
        legend_names{count} = 'Start bot grow circ, H1';
        count = count + 1;   
    elseif ID(j) == 10
        coreFun = CS_Enc;
        legend_names{count} = 'Start enc, grow circ, H3';
        count = count + 1;   
    end
    
    for i = 2:length(crack_array)
        funEq = @(x) 1./...
            (C.*...
            ((S.*sqrt(pi.*x).*...
            (coreFun(x)))-Kth).^m);
        
        Cycles_quadgk = quadgk(funEq, crack_array(i-1), crack_array(i));
        Cycles_Step = [Cycles_Step, Cycles_quadgk];
    end
    Cycles_Cum = [0,cumsum(Cycles_Step)];
    Cycles_Max = [Cycles_Max, Cycles_Cum(length(Cycles_Cum))];
%     subplot(1,2,1)
    plot(Cycles_Cum,crack_array./D,line_styles{j},'LineWidth',1.2)
    % % Reinitialize
    Cycles_Cum = 0;
    Cycles_Step = [];
    hold on
end

grid on
xlabel('Number of Cycles','FontSize',13)
ylabel('Crack Depth, a/D','FontSize',15)
title('Life Estimates for Chain Link','FontSize',14)
leg = legend(legend_names,'Location','northeastoutside');
set(leg,'FontSize',15);

diff_percent_biggest = 100-((max(Cycles_Max)-Cycles_Max)./max(Cycles_Max)).*100
local =   1.0e+04 * [2.2638    2.7714    0.6172];
 local =   1.0e+04 *[ 4.6404    3.6884    1.4362]% 5mm
SH2 = (1-(Cycles_Max(1)-local(1))/Cycles_Max(1))*100
SH2M = (1-(Cycles_Max(2)-local(1))/Cycles_Max(2))*100
BH1 = (1-(Cycles_Max(3)-local(2))/Cycles_Max(3))*100
EH3 = (1-(Cycles_Max(4)-local(3))/Cycles_Max(4))*100