clear
clc
% % % Input parameters
% Geometry
d = 0.076;

% Crack Growth
C = 17.5*10^-12;
m = 3;
Kth = 0;
% Loading
S = 100;



% ID indicates which configuartions will be added to the plot. 
ID = [1,2,3];
% % Style defintiions
line_styles = {'-b*','-rs','-k^','--bx','--rp','--oc','-.b>','-.r<','-.h'};
legend_names = {};
% % Looping function initializer
Cycles_Step = [];
Cycles_Cum = 0;
Cycles_Max = [];


% % % Crack
num_steps = 15; % How many intervals will be plotted
ai = 0.005; % Initial crack depth
af = 0.038; % Final crack depth
step = (af-ai)/num_steps; 
crack_array = ai:step:af;

count = 1;
for j =1:length(ID)
    if ID(j) == 1
        X1 = 4.4612;
        X2 = 1.1779;
        X3 = 0.1771;
        X4 = 2.7332;
        funChainC = @(x) (X1.*(x./d).^3 + X2.*(x./d).^2 + X3.*(x./d) + X4);
        coreFun = funChainC;
        legend_names{count} = 'Chain, start circular';
        count = count + 1;
    elseif ID(j) == 2
        const = [-9.9692   20.2699   -7.3719    3.5849];
        funChainE = @(x) const(1).*(x./d).^3+const(2).*(x./d).^2+const(3).*(x./d)+const(4);
        coreFun = funChainE;
        legend_names{count} = 'Chain, start elliptic';
        count = count + 1;
    elseif ID(j) == 3
        const = [3.1572    1.5817   -1.8467    2.7171];
        funChainPC = @(x) const(1).*(x./d).^3+const(2).*(x./d).^2+const(3).*(x./d)+const(4);
        coreFun = funChainPC;
        legend_names{count} = 'Chain, constant circular';
        count = count + 1;
    end
    
    for i = 2:length(crack_array)
        funEq = @(x) 1./...
            (C.*...
            ((S.*sqrt(pi.*x).*...
            (coreFun(x)))-Kth).^m);
        
        Cycles_quadgk = quadgk(funEq, crack_array(i-1), crack_array(i));
        Cycles_Step = [Cycles_Step, Cycles_quadgk];
    end
    Cycles_Cum = [0,cumsum(Cycles_Step)];
    Cycles_Max = [Cycles_Max, Cycles_Cum(length(Cycles_Cum))];
    plot(Cycles_Cum,crack_array./d,line_styles{j},'LineWidth',1.2)
    % % Reinitialize
    Cycles_Cum = 0;
    Cycles_Step = [];
    hold on
end

grid on
xlabel('Number of Cycles','FontSize',13)
ylabel('Crack Depth, a/D','FontSize',15)
title('Life Estimates for Crack Configurations','FontSize',14)
leg = legend(legend_names,'Location','northeastoutside');
set(leg,'FontSize',15);
diff_percent_biggest = (1-(max(Cycles_Max)-Cycles_Max)./max(Cycles_Max)).*100
