function [a, ang, K] = handbook_solution_tension(a,acRatio,ang,B,S,T) 
%AN EMPIRICAL STRESS-INTENSITY FACTOR EQUATION FOR THE SURFACE CRACK
    aT = a./T;
    M1 = 1.13-0.09*acRatio;
    M2 = -0.54+(0.89/(0.2+acRatio));
    M3 = 0.5-(1/(0.65+acRatio))+14*(1-acRatio)^24;
    g = 1+(0.1+0.35.*aT.^2).*(1-(sin(ang)).^2);
    f_phi = sqrt((acRatio).^2.*(cos(ang).^2)+(sin(ang)).^2);
    f_w = sqrt(sec(pi.*(a./acRatio)./(2.*B).*sqrt(a./T)));
    F_D = (M1+M2.*aT.^2+M3.*aT.^4).*f_phi.*f_w.*g;
    Q = 1+1.464*acRatio.^1.65;

    p = 0.2+acRatio+0.6.*a./T;
    G21 = -1.22-0.12.*acRatio;
    G22 = 0.55-1.05.*(acRatio).^0.75+0.47.*(acRatio).^1.5;
    H1 = 1-0.34.*a./T-0.11.*acRatio.*a./T;
    H2 = 1 + G21.*a./T + G22.*(a./T).^2;
    H_s = H1+(H2-H1).*sin(ang).^p;

    K = H_s.*F_D;
    length(K);
end