from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from abaqusConstants import * 
from time import sleep
import math
import numpy as np
import regionToolset

#def PartGenerator(crack_length):
# # # # IMPORTING 2D CHAIN PART # # # #

crackDistribution = np.arange(5,39,1)
for j in range(0,len(crackDistribution)):
    crack_length=crackDistribution[j]
    core = 'crckLn_2D_'
    modelName = core +str(crack_length)
    job_name = modelName
    myModel = mdb.Model(name=modelName, objectToCopy=mdb.models['Source'])    
    myPart = myModel.parts['Chain_Link']    
    elemPerMm = 1
    elemBox = 10;
    box = 10
    dist_req = 10
    
    typeOne = 2*box+dist_req
    # # # # Partition Sketch
    tempSketch = myModel.ConstrainedSketch(name='__profile__', sheetSize=200.0)
    
    bottom_right = np.array((-125.08,197.537005))
    upper_right = np.array((-102.426899,270.158391))
    top_crown = np.array((-140.08,275.726933))
    crack_front = top_crown - np.array((0.0,crack_length))
    
    originX = crack_front[0]
    originY = crack_front[1]
    
    local_upper = upper_right-crack_front
    local_lower = bottom_right-crack_front
    
    tempSketch.rectangle(point1=(-1,box),
                       point2=(box,0))                   
    tempSketch.rectangle(point1=(-1,0),
                       point2=(box,-box))
    
    
    tempSketch.Line(point1=(box,-box), 
                  point2=(local_lower[0],local_lower[1]))
    if crack_length<typeOne:
        tempSketch.Line(point1=(box,box),
                      point2=(box,100))
        #
        
    elif crack_length>typeOne-1:
        tempSketch.Line(point1=(box,box),
                     point2=(local_upper[0],local_upper[1]))
    
    
    myModel.ConstrainedSketch(name='Partition_Sketch_2', objectToCopy=
        tempSketch)
    del tempSketch
    
    
    myModel.ConstrainedSketch(gridSpacing=4.79, name='__profile__', 
        sheetSize=200.0, transform=
        myPart.MakeSketchTransform(
        sketchPlane=myPart.faces[0], 
        sketchPlaneSide=SIDE1, sketchOrientation=RIGHT, origin=(top_crown[0], 
       top_crown[1]-crack_length, 0.0)))
        
    #    
    #myPart.projectReferencesOntoSketch(filter=
    #    COPLANAR_EDGES, sketch=myModel.sketches['__profile__'])
        
    myModel.sketches['__profile__'].retrieveSketch(sketch=
        myModel.sketches['Partition_Sketch_2'])
    
    
    myPart.PartitionFaceBySketch(faces=
        myPart.faces.findAt(((top_crown[0]+1,top_crown[1]-5,0.0),),),
                 sketch=myModel.sketches['__profile__'])
    
    del myModel.sketches['__profile__']
    
    # # # # COMMON GEOMETRY LINES
    lowerLine1 = myPart.edges.getClosest(coordinates=
                    ((originX+box+local_lower[0]/2,originY-box+local_lower[1],0.0),))[0][0]
    lowerLine1 = (myPart.edges[lowerLine1.index],)
    lowerLine2 = myPart.edges.getClosest(coordinates=
                    ((originX,originY-1.5*box,0.0),))[0][0]
    lowerLine2 = (myPart.edges[lowerLine2.index],)
    bottomLine = myPart.edges.findAt(((-129.864294,198.728909,0.),))
    upperRight = myPart.edges.findAt(((-108.13546,271.904134,0),))
    diagonalBound = myPart.edges.findAt(((-103.731081,229.795184,0.),))
    lowerDiagonal = myPart.edges.getClosest(coordinates=((originX+box+local_lower[0]/2,
                                          originY-box+local_lower[1]/2,
                                          0.0),))[0][0]
    lowerDiagonal = (myPart.edges[lowerDiagonal.index],)
    lowerPart = myPart.edges.findAt(((-90.272532,105.253645,0),))
    
    allFaces = myPart.faces.getByBoundingBox(-1000,-1000,0.0,
                                             1000,1000,0.0)
    
    
    
    # # # # AJSGKASFJSAP
    myRoot = myModel.rootAssembly
    myInst = myRoot.instances['Chain_Link-1']
    instCrownTop = 115.630108
    
    
     # # # MESHING 
    if crack_length<box+1:
        nElem = abs(crack_length/elemPerMm)
    
        # # # # Edges with 10 elements # # # #
    
        lowerBoxBottom = myPart.edges.findAt(((originX+box/2,originY-box,0.0),))
        lowerBoxTop = myPart.edges.findAt(((originX+box/2,originY,0.0),))
        lowerBoxRight = myPart.edges.findAt(((originX+box,originY-box/2,0.0),))
    
                                                       
        lowerBoxLeft = myPart.edges.getClosest(coordinates=
                                                ((originX,originY-box/2,0.0),))[0][0]                                                    
        lowerBoxLeft = (myPart.edges[lowerBoxLeft.index],)
        
        
        aboveBoxRight = myPart.edges.getClosest(coordinates=
                                                ((originX+box,
                                                  originY+crack_length/2,
                                                  0.0),))[0][0]
        aboveBoxRight = myPart.edges[aboveBoxRight.index]
        aboveBoxLeft = myPart.edges.getClosest(coordinates=((originX,
                                                             originY+crack_length/2,
                                                             0.0),))[0][0]
        aboveBoxLeft = myPart.edges[aboveBoxLeft.index]
                                    
                                    
        aboveBoxTop = myPart.edges.findAt(((-135.051889,275.560067,0.),))   
        
        
        myPart.Set(name='Elem_10',
                  edges=(lowerBoxBottom,lowerBoxTop,lowerBoxRight,
                   bottomLine,aboveBoxTop))
    
        myPart.seedEdgeByNumber(edges=lowerLine1, 
                                number=15)
        myPart.seedEdgeByNumber(edges=lowerLine2, 
                                number=15)
        myPart.seedEdgeByNumber(edges=upperRight, 
                                number=15)
        myPart.seedEdgeByNumber(edges=lowerDiagonal,
                                number=15)                
        myPart.seedEdgeByNumber(edges=diagonalBound,
                                number=(2*elemBox+nElem))
        myPart.seedEdgeByNumber(edges=myPart.sets['Elem_10'].edges,
                                number=elemBox)           
        myPart.seedEdgeByNumber(edges=lowerBoxLeft,
                                number=elemBox)    
        myPart.seedEdgeByNumber(edges=(aboveBoxLeft,), 
                                number=nElem)
        myPart.seedEdgeByNumber(edges=(aboveBoxRight,), 
                                number=nElem)
        myPart.seedEdgeBySize(edges=(lowerPart),
                              size=8) 
        myPart.Set(faces=(allFaces),name='All Faces')
        myPart.setElementType(regions=myPart.sets['All Faces'], elemTypes=(CPE8,CPE6M) )
        myPart.setMeshControls(regions=(allFaces), elemShape=QUAD,
                               technique=STRUCTURED)
        myPart.generateMesh()
        
        
        # CRACK PARAMETERS #
        crackVertice = regionToolset.Region(vertices = myInst.vertices.findAt(((
                                        0,instCrownTop-crack_length,0.0),))
                                        )
        crackFace = regionToolset.Region(edges=myInst.edges.findAt(((0.0,instCrownTop-0.5,0.0),),))
                                                        
    elif ((crack_length<typeOne) and (crack_length>box)):
        # # # # Edges with 10 elements # # # #
        nElem = abs(crack_length-box)
        lowerBoxBottom = myPart.edges.findAt(((originX+box/2,originY-box,0.0),))
        lowerBoxTop = myPart.edges.findAt(((originX+box/2,originY,0.0),))
        lowerBoxRight = myPart.edges.findAt(((originX+box,originY-box/2,0.0),))
                                                   
        lowerBoxLeft = myPart.edges.getClosest(coordinates=
                                                ((originX,originY-box/2,0.0),))[0][0]                                                    
        lowerBoxLeft = (myPart.edges[lowerBoxLeft.index],)
        
        upperBoxTop = myPart.edges.findAt(((originX+box/2,originY,0.0),))    
        upperBoxLeft = myPart.edges.findAt(((originX, originY+box/2,0.0,),))
        upperBoxRight = myPart.edges.findAt(((originX+box,originY+box/2,0.0),))
        
        aboveBoxRight = myPart.edges.getClosest(coordinates=
                                                ((originX+box,
                                                  originY+box+crack_length/2,
                                                  0.0),))[0][0]
        aboveBoxRight = myPart.edges[aboveBoxRight.index]
        
        aboveBoxLeft = myPart.edges.getClosest(coordinates=((originX,
                                                             originY+box+crack_length/2,
                                                             0.0),))[0][0]
        aboveBoxLeft = myPart.edges[aboveBoxLeft.index]
                                    
                                    
        aboveBoxTop = myPart.edges.findAt(((-135.051889,275.560067,0.),))   
        
        
        myPart.Set(name='Elem_10',
                  edges=(lowerBoxBottom,lowerBoxTop,lowerBoxRight,
                   bottomLine,aboveBoxTop))
    
    
        myPart.seedEdgeByNumber(edges=lowerLine1, 
                                number=15)
        myPart.seedEdgeByNumber(edges=lowerLine2, 
                                number=15)
        myPart.seedEdgeByNumber(edges=upperRight, 
                                number=15)
                                
                                
        myPart.seedEdgeByNumber(edges=lowerDiagonal,
                                number=15)                
        myPart.seedEdgeByNumber(edges=diagonalBound,
                                number=(2*elemBox+nElem))
                                
                                
        myPart.seedEdgeByNumber(edges=myPart.sets['Elem_10'].edges,
                                number=elemBox)           
        myPart.seedEdgeByNumber(edges=lowerBoxLeft,
                                number=elemBox)    
                                
                                
        myPart.seedEdgeByNumber(edges=upperBoxRight,
                                number=10)  
        myPart.seedEdgeByNumber(edges=upperBoxLeft,
                                number=10)        
        myPart.seedEdgeByNumber(edges=upperBoxTop,
                                number=10)      
                                
                                
        myPart.seedEdgeByNumber(edges=(aboveBoxLeft,), 
                                number=nElem)
        myPart.seedEdgeByNumber(edges=(aboveBoxRight,), 
                                number=nElem)
                                
                                
        myPart.seedEdgeBySize(edges=(lowerPart),
                              size=8) 
    
        myPart.Set(faces=(allFaces),name='All Faces')
        myPart.setElementType(regions=myPart.sets['All Faces'], elemTypes=(CPE8,CPE6M) )
        myPart.setMeshControls(regions=(allFaces), elemShape=QUAD,
                               technique=STRUCTURED)
        myPart.generateMesh()
    
        
        # CRACK PARAMETERS #
        crackVertice = regionToolset.Region(vertices = myInst.vertices.findAt(((
                                        0,instCrownTop-crack_length,0.0),))
                                        )
        crackFace = regionToolset.Region(edges=myInst.edges.findAt(((0.0,instCrownTop-0.5,0.0),),
                                                                   ((0.0,instCrownTop-crack_length+0.5,0.0),),))
    
    
    elif (crack_length>typeOne-1):
    
        # # # # Edges with 10 elements # # # #
        nElem = abs(crack_length-box)
        lowerBoxBottom = myPart.edges.findAt(((originX+box/2,originY-box,0.0),))
        lowerBoxTop = myPart.edges.findAt(((originX+box/2,originY,0.0),))
        lowerBoxRight = myPart.edges.findAt(((originX+box,originY-box/2,0.0),))
                                                   
        lowerBoxLeft = myPart.edges.getClosest(coordinates=
                                                ((originX,originY-box/2,0.0),))[0][0]                                                    
        lowerBoxLeft = (myPart.edges[lowerBoxLeft.index],)
        
        upperBoxTop = myPart.edges.findAt(((originX+box/2,originY,0.0),))    
        upperBoxLeft = myPart.edges.findAt(((originX, originY+box/2,0.0,),))
        upperBoxRight = myPart.edges.findAt(((originX+box,originY+box/2,0.0),))
        
        aboveBoxRight = myPart.edges.getClosest(coordinates=
                                                ((originX+box,
                                                  originY+box+crack_length/2,
                                                  0.0),))[0][0]
        
        aboveBoxLeft = myPart.edges.getClosest(coordinates=((originX,
                                                             originY+box+crack_length/2,
                                                             0.0),))[0][0]
        aboveBoxLeft = myPart.edges[aboveBoxLeft.index]
                                    
                                    
        aboveBoxTop = myPart.edges.findAt(((-135.051889,275.560067,0.),))   
        
        
        myPart.Set(name='Elem_10',
                  edges=(lowerBoxBottom,lowerBoxTop,lowerBoxRight,
                   bottomLine,aboveBoxTop))
    
    
        myPart.seedEdgeByNumber(edges=lowerLine1, 
                                number=15)
        myPart.seedEdgeByNumber(edges=lowerLine2, 
                                number=15)
        myPart.seedEdgeByNumber(edges=(aboveBoxLeft,), 
                                number=15)                            
                                
        myPart.seedEdgeByNumber(edges=lowerDiagonal,
                                number=15)                
        myPart.seedEdgeByNumber(edges=diagonalBound,
                                number=(2*elemBox))
                                
                                
        myPart.seedEdgeByNumber(edges=myPart.sets['Elem_10'].edges,
                                number=elemBox)           
        myPart.seedEdgeByNumber(edges=lowerBoxLeft,
                                number=elemBox)    
        myPart.seedEdgeByNumber(edges=upperRight, 
                                number=elemBox)                            
                                
        myPart.seedEdgeByNumber(edges=upperBoxRight,
                                number=elemBox)  
        myPart.seedEdgeByNumber(edges=upperBoxLeft,
                                number=elemBox)        
        myPart.seedEdgeByNumber(edges=upperBoxTop,
                                number=elemBox)      
    
       
        myPart.seedEdgeBySize(edges=(lowerPart),
                              size=8) 
    
        myPart.Set(faces=(allFaces),name='All Faces')
        myPart.setElementType(regions=myPart.sets['All Faces'], elemTypes=(CPE8,CPE6M) )
        
        # # # SETTING MESH CONTROLS # # #
        
        myPart.setMeshControls(regions=(allFaces), elemShape=QUAD,
                               technique=STRUCTURED)
        top_face = myInst.faces.findAt(((1,instCrownTop-5,0.0),))
        myPart.setMeshControls(regions=(top_face), elemShape=QUAD,
                       technique=SWEEP)                       
        myPart.generateMesh()
    
        # CRACK PARAMETERS #
    myPart.regenerate()
    myRoot.regenerate()
    crackVertice = regionToolset.Region(vertices = myInst.vertices.findAt(((
                                    0,instCrownTop-crack_length,0.0),))
                                    )
    crackFace = regionToolset.Region(edges=myInst.edges.findAt(((0.0,instCrownTop-crack_length-box/20.0,0.0),),
                                                               ((0.0,instCrownTop-crack_length+box/20.0,0.0),),))
        
    # # # # BC # # # #
    lowerLine2 = myInst.edges.findAt(((0.0,instCrownTop-crack_length-box/2,0.0),))
    lowerBoxLeft = myInst.edges.findAt(((0.0,50.0,0.0),))                                                
        
    myRoot.Set(edges=
        lowerLine2, name='lowestSym')
    
    myRoot.Set(edges=
        lowerBoxLeft, name='lowestBoxSym')
        
    myModel.XsymmBC(createStepName='Initial', localCsys=None, name='symmetry',
                region=myRoot.sets['lowestSym'])
    myModel.XsymmBC(createStepName='Initial', localCsys=None, name='symmetryBox',
                region=myRoot.sets['lowestBoxSym'])
                
                
                
    ## INSERTING CRACK #
    myRoot.engineeringFeatures.ContourIntegral(
        collapsedElementAtTip=NONE, crackFront=crackFace, 
        crackTip=crackVertice,
        extensionDirectionMethod=Q_VECTORS, midNodePosition=0.25, 
        name='crack', qVectors=(((0.0, 0.0, 0.0), (0.0, -1.0, 0.0)), ), 
        symmetric=ON)
    
    ## # # CREATING OUTPUT # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    ## CRACK OUPUT #
    myModel.HistoryOutputRequest(contourIntegral='crack', 
        createStepName='General', name='J_Integral', numberOfContours=10, rebar=EXCLUDE, 
        sectionPoints=DEFAULT)
        
    myModel.FieldOutputRequest(createStepName='General', name='F-Output-2'
    , variables=('S', 'MISES', 'E', 'U', 'UR', 'RF'))
    
    myModel.HistoryOutputRequest(createStepName='General', name=
    'H-Output-2', variables=PRESELECT)
    
    myModel.HistoryOutputRequest(contourIntegral='crack', 
    contourType=K_FACTORS, createStepName='General', frequency=LAST_INCREMENT, 
    name='SIF_factors', numberOfContours=box, rebar=EXCLUDE, sectionPoints=
    DEFAULT)
    
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
        explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
        memory=95, memoryUnits=PERCENTAGE, model=modelName, modelPrint=OFF, 
        multiprocessingMode=DEFAULT, name=job_name, nodalOutputPrecision=
        SINGLE, numCpus=4, numGPUs=4, numDomains=4, queue=None, resultsFormat=ODB
        , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)

    
for i in range(0,len(crackDistribution)):
    jobName = core+str(crackDistribution[i])
    mdb.jobs[jobName].submit(consistencyChecking=OFF)
    mdb.jobs[jobName].waitForCompletion()    
#    
#    
    