# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# crack_array - The range of crack lengths analyzed (linked to file names)
# odbSource - Location of ODB files
# saveDest - Destination to save the jIntegral maximum values as npy files
# saveDestcsv - Destination to save the csv files with max jIntegral and 
#               another file which indicates which picked set loc has max J
# core - Core name of the files that are looped (crack length in name varies)
# nrContour - Number of contours asked for in the analysis.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
import math
import numpy as np
import odbAccess as odb

jValues = []
jMax = []
maxSetLoc = []

crack_short = np.arange(5,10,1)
crack_long = np.arange(10,40,2)
crack_array = np.concatenate([crack_short,crack_long])

crackLength = crack_array


odbSource = 'D:/0_MASTER/ABAQUS/Evy_Crack/ODB/'
saveDest = 'D:/0_MASTER/ABAQUS/Evy_Crack/NPY/'
saveDestcsv = 'D:/0_MASTER/ABAQUS/Evy_Crack/CSV/'
core_name = '76mm_100MPa_'
core = odbSource + core_name


nrContour = 5
maxContour = str(nrContour)


for j in range (0,len(crackLength)):
    if crackLength[j]<10:
        pickedSet = str(183)
        nrContour = 5
        maxContour = str(nrContour)
    elif crackLength[j]<24:
        pickedSet = str(263)
        nrContour = 10
        maxContour = str(nrContour)
    else:
        pickedSet = str(287)
        nrContour = 5
        maxContour = str(nrContour)       
    
    jobName = core+str(crackLength[j])+'.odb'
    odbPath = jobName
    print(odbPath)
    od = odb.openOdb(path=odbPath)
    nrSubSet = (len(od.rootAssembly.nodeSets)-1)/nrContour-4 # Originally -1, but reduced to -5, as there are 4 other sets present in the model. 
    myStep = od.steps['General'] 
        
    for k in range(1,nrSubSet+1):
        # # # # Get J-INTEGRAL
        jIntLast = myStep.historyRegions['ElementSet . ALL ELEMENTS'].historyOutputs['J at CRACK_CRACK-1__PICKEDSET'+pickedSet+'-'+str(k)+'__Contour_'+maxContour].data[0][1]
        jValues.append(jIntLast)
    
    
# # # # FINDING COORDINATES AND ANGLES OF EACH POINT ON CRACK LINE # # # # # #   
    myRoot=od.rootAssembly    
    labelList = []
    angleRad =  []

    
    originX = 0.0
    originY = 76.0
    crackLen = crackLength[j]
    for i in range(1,nrSubSet+1):
        coordList=[]
        if crackLength[j]<10:
            nod = myRoot.nodeSets['CRACK_CRACK-1_J__PICKEDSET'+pickedSet+'-'+str(i)+'__Contour_1'].nodes[0]
        elif crackLength[j]<24:
            nod = myRoot.nodeSets['CRACK_CRACK-1_J__PICKEDSET'+pickedSet+'-'+str(i)+'__Contour_01'].nodes[0]
        else:
            nod = myRoot.nodeSets['CRACK_CRACK-1_J__PICKEDSET'+pickedSet+'-'+str(i)+'__Contour_1'].nodes[0]
        
        length = len(nod)
        radiusContour = []
        labelContour = []
        ySaver = []
        degChecker = []
        for i in range(0,length):
            trgNod = nod[i]
            trgCoord = nod[i].coordinates
            x = trgCoord[0]-originX
            y = trgCoord[1]-originY
            radius = math.sqrt(math.pow(x,2)+math.pow(y,2))
            
            if x<0:
                degChecker.append(1)
            else:
                degChecker.append(0)
            
            ySaver.append(y)
            radiusContour.append(radius)
            labelContour.append(nod[i].label)
        
        sortRadius = np.sort(radiusContour,)
        indexSort = np.argsort(radiusContour)
        index = int((len(sortRadius)-1)/2)
        centerLabel = labelContour[indexSort[index]]
        
        yArray = np.array(ySaver)
        yOrdered = yArray[indexSort]
        
        degCheckerArray = np.array(degChecker)
        degCheckerCenter = degChecker[indexSort[index]]
        yCenter = yOrdered[index]
        labelList.append(centerLabel)
        angle = -(math.pi/2+math.asin(yCenter/crackLen))
    
        if degCheckerCenter==1:
            angle=-(math.pi+angle)
        angleRad.append(angle)
    
    angleDeg = np.array(angleRad)*180.0/math.pi
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #    
    
    
    
    
    od.close()            

    jFinal = np.array(jValues)
    
    crackFace = np.concatenate(([abs(jFinal)],[angleDeg]),0)
    np.savetxt(saveDestcsv+'Chain_crack_length'+str(crackLength[j])+'.csv',
               crackFace)  
    
    
    
    tempMax = max(jFinal)
    np.save(saveDest+'j_integral_chain_'+str(crackLength[j]),jFinal)
    tempLoc = np.nonzero((jFinal==tempMax))[0][0]+1

    jValues = []
    
    maxSetLoc.append(tempLoc)
    jMax.append(abs(tempMax))

# Save npy files that can later be converted to .mat    
np.save(saveDest+'j_max_Loc_Chain_ALL',maxSetLoc)
np.save(saveDest+'j_max_Chain_ALL',jMax)

# Save as CSV files, which can be opened by excel
np.savetxt(saveDestcsv+'j_max_Loc_Chain_ALL.csv',maxSetLoc,delimiter=',')
np.savetxt(saveDestcsv+'j_max_Chain_ALL.csv',jMax,delimiter=',')

