from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from abaqusConstants import *

import numpy as np
import math

# Dimensions 
cubeX = 20.0
cubeY = 20.0
cubeZ = 20.0
notch_radius = 1.0



# Mesh things
nrInc = 10
seedContain=30
seedCrackFront = 20
nrLines = 32
degChanges = 360.0/nrLines
global_seed_size = 0.8 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
seedNotchEdge = 15

xBuffer = 10/30.0
yBuffer = 10/30.0
crack_mesh_radius = 0.15
crackBuffer = 3*crack_mesh_radius/nrInc


crack_array=np.arange(1.4,1.4,0.1)
crack_array=np.array([1.4])
for l in range(0,len(crack_array)):
    nrInc = 5
    seedContain=30
    seedCrackFront = 20
    nrLines = 16
    degChanges = 360.0/nrLines
    global_seed_size = 0.8 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 8
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
        
        
        
    
    
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    core_name = 'Notch_Bot_E_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'ref'
    
    myModel = mdb.Model(name=core_name)

    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    cutSketch = myModel.ConstrainedSketch(name='cube_cut', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = myModel.ConstrainedSketch(name='cut_raw', sheetSize=300,
                                       transform=cube.MakeSketchTransform(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX,cubeY,cubeZ)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cube.CutRevolve(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketch
    

    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep





    # # # CREATING CRACK CIRCLE

    containShape = myModel.ConstrainedSketch(name='contain_shape', sheetSize=200.0) 
    innerCrack = crack_depth-crackBuffer-crack_mesh_radius
    outerCrack = crack_depth+crackBuffer+crack_mesh_radius
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    
    containShape.Line(point1=(0,outerCrack), point2=(0,-(outerCrack)))
    containShape.Line(point1=(0,outerCrack), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(innerCrack),0)),
                            point1=(-(innerCrack),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    myModel.Part(dimensionality=THREE_D, name='Contain', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Contain'].BaseShellRevolve(angle=360, sketch=
            containShape)




    # CUTTING
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])
    
    center = (cubeX,cubeY,cubeZ)
    
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.translate(instanceList=['Crack',], vector=center)

    myRoot.Instance(name='Contain', part=myModel.parts['Contain'])
    myRoot.translate(instanceList=['Contain',], vector=center)    
                              
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Contain'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)    
    myPart = myModel.parts['Cracked_Notch']


    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((0.01,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((0.01,cubeY/2,0),))[0]
        
        #
#    x1 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX-(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
#    x2 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX+(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
    #    z1 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2)
#    y1 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY-(crack_depth+yBuffer+crackBuffer)), flip=SIDE2)
        
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[x1.id])    
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[x2.id])                                                    
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[z1.id])     
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[y1.id])                                                         
                                                        
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    myRoot.makeIndependent(instances=(myInst,))
    
    
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # CRACK FRONT
#    seedCrackFront = 16
#    originX = cubeX
#    originY = cubeY
#    originZ = cubeZ
#    myRoot.seedEdgeByNumber(number=seedCrackFront,
#                            edges=(myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
#                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
#                                                         cubeZ),))[0],))
#                                                         
#    crackEdge = myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
#                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
#                                                         cubeZ),))                                                     
#                                                         
    #    
    originX = cubeX - crack_depth
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = (degChanges)*math.pi/180*(0.5+i)
    	degLine = (degChanges)/180*math.pi*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)    
        
    
     # # # SEEDS FOR CRACK BOX
    degRad = degChanges/2*math.pi/180.0
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    

    degRad = 1*math.pi/180

    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=nrLines/8)        
    # # # SEEDS ALONG OUTER CIRCLE
    degRad = 3*degChanges*math.pi/180.0 
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=seedContain)        
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=seedContain)    
                            
                            
    innerDivider = innerCrack
    outerDivider = outerCrack
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
                            
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*innerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*innerDivider),))[0],))
    
    symEdgeX=myInst.edges.findAt(((cubeX,cubeY-notch_radius*math.sqrt(0.5),cubeZ-notch_radius*math.sqrt(0.5)),))[0]
    symEdgeZ=myInst.edges.findAt(((cubeX-notch_radius*math.sqrt(0.5),cubeY-notch_radius*math.sqrt(0.5),cubeZ),))[0]
    myRoot.seedEdgeByNumber(edges=(symEdgeX,),number=seedNotchEdge)     
    myRoot.seedEdgeByNumber(edges=(symEdgeZ,),number=seedNotchEdge)
    # # # CRACK FRONT
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    myRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    crackEdge = myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))

                                                     
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size)                                                        
#

    
    
 



#   # # CREATING STEP    
    
    myModel.StaticStep(name='General', previous='Initial')
    
    # # # DEFINING CRACK  
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = cubeY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    #    
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    
    
    
    myRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=crackEdge), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.25, name='Crack-1', symmetric=ON)
    
    
    
    
    
    # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    
    myModel.ExpressionField(description='', 
                            expression='(Y-'+str(cubeY/2)+')/'+str(cubeY/2), localCsys=None, name='Bending Function')
#    mdb.models['Notch_Bot_E_D1_L13_ref'].loads['Loading'].setValues(
#    )
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=FIELD, 
                     field='Bending Function', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 
    
    
    
    
    
    # # # BOUNDARY CONDITIONS
    # underneath crack front
    underCrackLeft = myInst.faces.findAt(((cubeX-(crack_depth+crack_mesh_radius+crackBuffer+0.01),
                                           cubeY-0.01, cubeZ),))
    myModel.ZsymmBC(createStepName='General', localCsys=None, name='Sym_Z_Under_Crack_Left',
    			region=Region(faces=underCrackLeft))

    xSymFace = myInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    myModel.XsymmBC(createStepName='General', localCsys=None, name='Sym_X',
    			region=Region(faces=xSymFace))

#    bottomFace = myInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
#    myModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
#                           region=Region(faces=bottomFace),
#                            u2=0)
   
    # CRACK FRONT SYMMETRIES
    originX = cubeX   
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_L_'+str(i)
    	myModel.ZsymmBC(createStepName='General', localCsys=None, name=bcName,
    			region=Region(faces=myInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))

    


    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
		memory=90, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
		multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
    mdb.jobs[core_name].submit()
    mdb.jobs[core_name].waitForCompletion()
    
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    
    
    
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines
    global_seed_size = 0.15
    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
        
    
    
    
    

    cubeX_Sub = 5.0
    cubeY_Sub = 5.0
    cubeZ_Sub = 5.0
                
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    sub_name = 'Notch_Bot_E_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
    
    
    mySubModel = mdb.Model(name=sub_name, objectToCopy=
            mdb.models[core_name]) 

    mySubModel.setValues(description='Submodel notched cube',
                         globalJob=core_name+'.odb')  



    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    
    # CREATING CUBE 
    cubeSketchSub = mySubModel.ConstrainedSketch(name='cube_Sub', sheetSize=300)
    cubeSketchSub.rectangle(point1=(0,0),point2=(cubeX_Sub,cubeY_Sub))
    
    mySubModel.Part(dimensionality=THREE_D, name='Cube_Sub', type=DEFORMABLE_BODY)
    
    cubeSub = mySubModel.parts['Cube_Sub']
    cubeSub.BaseSolidExtrude(sketch=cubeSketchSub, depth=cubeZ_Sub)
    
    cutSketch = mySubModel.ConstrainedSketch(name='cube_cut_Sub', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = mySubModel.ConstrainedSketch(name='cut_raw_Sub', sheetSize=300,
                                       transform=cubeSub.MakeSketchTransform(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX_Sub,cubeY_Sub,cubeZ_Sub)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cubeSub.CutRevolve(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketchSub
    

    
    # # # CREATING CRACK SHELL
    crackSweep = mySubModel.ConstrainedSketch(name='crack_sweep_Sub', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = mySubModel.ConstrainedSketch(name='crack_shape_Sub', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    mySubModel.Part(dimensionality=THREE_D, name='Crack_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Crack_Sub'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep





    # # # # CREATING CRACK CIRCLE

    containShape = mySubModel.ConstrainedSketch(name='contain_shape_Sub', sheetSize=200.0) 
    innerCrack = crack_depth-crackBuffer-crack_mesh_radius
    outerCrack = crack_depth+crackBuffer+crack_mesh_radius
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    
    containShape.Line(point1=(0,outerCrack), point2=(0,-(outerCrack)))
    containShape.Line(point1=(0,outerCrack), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(innerCrack),0)),
                            point1=(-(innerCrack),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    mySubModel.Part(dimensionality=THREE_D, name='Contain_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Contain_Sub'].BaseShellRevolve(angle=360, sketch=
            containShape)




    # CUTTING
    centerX = cubeX-cubeX_Sub
    centerY = cubeY-cubeY_Sub
    centerZ = cubeZ-cubeZ_Sub
    centerCube = (centerX,centerY,centerZ)
    center = (cubeX,cubeY,cubeZ)
    
    
    
    mySubRoot = mySubModel.rootAssembly

    
    
    mySubRoot.Instance(name='Cube_Sub',part=mySubModel.parts['Cube_Sub'])
    mySubRoot.translate(instanceList=['Cube_Sub',], vector=centerCube)
    
    
    
    mySubRoot.Instance(name='Crack_Sub', part=mySubModel.parts['Crack_Sub'])
    mySubRoot.translate(instanceList=['Crack_Sub',], vector=center)

    mySubRoot.Instance(name='Contain_Sub', part=mySubModel.parts['Contain_Sub'])
    mySubRoot.translate(instanceList=['Contain_Sub',], vector=center)    
                              
    mySubInst = mySubRoot.InstanceFromBooleanCut(name='Cracked_Notch_Sub', 
                                  cuttingInstances=(mySubRoot.instances['Contain_Sub'],mySubRoot.instances['Crack_Sub'],),
                                  instanceToBeCut=mySubRoot.instances['Cube_Sub'], 
                                  originalInstances=SUPPRESS)    
    mySubPart = mySubModel.parts['Cracked_Notch_Sub']

    del mySubRoot.features['Cracked_Notch-1']
    del mySubModel.steps['General']        
    
    
    
    # # #Datum Planes
    faceX = mySubPart.faces.findAt(((cubeX,-cubeY_Sub/2+cubeY,cubeZ-cubeZ_Sub/2),))[0]
    faceY = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub,cubeZ-cubeZ_Sub/2),))[0]
    faceZ = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub/2,cubeZ),))[0]
        

    z2 = mySubPart.DatumPlaneByOffset(plane=faceZ, offset=((crack_mesh_radius+crackBuffer)), flip=SIDE2)

    
    mySubPart.PartitionCellByDatumPlane(cells=mySubPart.cells[0:len(mySubPart.cells)], 
                                                        datumPlane=mySubPart.datums[z2.id]) 
                                                        
                                                        
    # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    mySubModel.Material(name='Steel')
    mySubModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    mySubModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    mySubPart.Set(cells=
        mySubPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    mySubPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        mySubPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    mySubRoot.makeIndependent(instances=(mySubInst,))
    
    
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # # CRACK FRONT
# #    seedCrackFront = 16
# #    originX = cubeX_Sub
# #    originY = cubeY_Sub
# #    originZ = cubeZ_Sub
# #    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
# #                            edges=(mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         cubeZ_Sub),))[0],))
# #                                                         
# #    crackEdge = mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         cubeZ_Sub),))                                                     
# #                                                         
    # #    
    originX = cubeX - crack_depth
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = (degChanges)*math.pi/180*(0.5+i)
    	degLine = (degChanges)/180*math.pi*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)    
        
    
    #SEEDS FOR CRACK BOX
    degRad = degChanges/2*math.pi/180.0
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    

    degRad = 1*math.pi/180

    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=nrLines/8)        
    # # SEEDS ALONG OUTER CIRCLE
    degRad = 3*degChanges*math.pi/180.0 
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=seedContain)        
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=seedContain)    
                            
                            
    innerDivider = innerCrack
    outerDivider = outerCrack
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
                            
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*innerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*innerDivider),))[0],))
                            
        # # # # CRACK FRONT
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    crackEdge = mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))
    
                                                     
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch_Sub-1'],), size=global_seed_size)                                                        
    # #
    
    
    
     
    
    
    
    # #   # # CREATING STEP    
    
    mySubModel.StaticStep(name='General_Sub', previous='Initial')
    
    # # # # DEFINING CRACK  
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = cubeY_Sub-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    #    
    crackSurf = mySubInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+mySubInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    
    
    
    mySubRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=crackEdge), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.25, name='Crack-1', symmetric=ON)
    
    
    
    
    
    # # # GENERATING MESH
    mySubRoot.setElementType(regions=(mySubInst.cells[0:len(mySubInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    mySubRoot.generateMesh((mySubInst,))
    
    
    # # # LOADING   
    
    
    
    # # # # BOUNDARY CONDITIONS
    # # underneath crack front
    underCrackLeft = mySubInst.faces.findAt(((cubeX-(crack_depth+crack_mesh_radius+crackBuffer+0.01),
                                           cubeY-0.01, cubeZ),))
    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_Under_Crack_Left',
    			region=Region(faces=underCrackLeft))
    
    xSymFace = mySubInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    mySubModel.XsymmBC(createStepName='Initial', localCsys=None, name='Sym_X',
    			region=Region(faces=xSymFace))
    
    # #  bottomFace = mySubInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
    # #  mySubModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
    # #                           region=Region(faces=bottomFace),
    # #                            u2=0)
       
    # # CRACK FRONT SYMMETRIES
    originX = cubeX   
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_L_'+str(i)
    	mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
    			region=Region(faces=mySubInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))
    
    
    
    # # SUBMODEL BC's
    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
                  mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
                  mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),)
    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
                
    region = mySubRoot.sets['subDrivenBoundary']
    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
        region=region, globalStep='1', 
        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3),
        globalDrivingRegion='', 
        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)      
    
    
    
    
    
    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
    		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    		memory=90, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
    		multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
    		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
    		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
                
    mdb.jobs[sub_name].submit()
    mdb.jobs[sub_name].waitForCompletion()                