from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from abaqusConstants import *

import numpy as np
import math

# Dimensions 
cubeX = 20.0
cubeY = 20.0
cubeZ = 20.0
notch_radius = 1.0



# Mesh things
nrInc = 10
seedContain=30
seedCrackFront = 20
nrLines = 32
degChanges = 360.0/nrLines
global_seed_size = 0.8 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
seedNotchEdge = 15

xBuffer = 10/30.0
yBuffer = 10/30.0
crack_mesh_radius = 0.15
crackBuffer = 3*crack_mesh_radius/nrInc


crack_array=np.arange(1.3,2.1,0.1)

for l in range(0,len(crack_array)):
    nrInc = 5
    seedContain=30
    seedCrackFront = 20
    nrLines = 16
    degChanges = 360.0/nrLines
    global_seed_size = 0.8 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 8
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
        
        
        
    
    
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    core_name = 'Notch_Bot_E_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'ref'
    
    myModel = mdb.Model(name=core_name)

    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    cutSketch = myModel.ConstrainedSketch(name='cube_cut', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = myModel.ConstrainedSketch(name='cut_raw', sheetSize=300,
                                       transform=cube.MakeSketchTransform(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX,cubeY,cubeZ)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cube.CutRevolve(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketch
    

    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep





    # # # CREATING CRACK CIRCLE

    containShape = myModel.ConstrainedSketch(name='contain_shape', sheetSize=200.0) 
    innerCrack = crack_depth-crackBuffer-crack_mesh_radius
    outerCrack = crack_depth+crackBuffer+crack_mesh_radius
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    
    containShape.Line(point1=(0,outerCrack), point2=(0,-(outerCrack)))
    containShape.Line(point1=(0,outerCrack), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(innerCrack),0)),
                            point1=(-(innerCrack),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    myModel.Part(dimensionality=THREE_D, name='Contain', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Contain'].BaseShellRevolve(angle=360, sketch=
            containShape)




    # CUTTING
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])
    
    center = (cubeX,cubeY,cubeZ)
    
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.translate(instanceList=['Crack',], vector=center)

    myRoot.Instance(name='Contain', part=myModel.parts['Contain'])
    myRoot.translate(instanceList=['Contain',], vector=center)    
                              
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Contain'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)    
    myPart = myModel.parts['Cracked_Notch']


    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((0.01,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((0.01,cubeY/2,0),))[0]
        
        #
#    x1 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX-(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
#    x2 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX+(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
    #    z1 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_depth+xBuffer+crackBuffer)), flip=SIDE2)
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2)
#    y1 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY-(crack_depth+yBuffer+crackBuffer)), flip=SIDE2)
        
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[x1.id])    
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[x2.id])                                                    
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[z1.id])     
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 
#    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
#                                                        datumPlane=myPart.datums[y1.id])                                                         
                                                        
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    myRoot.makeIndependent(instances=(myInst,))
    
    
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # CRACK FRONT
#    seedCrackFront = 16
#    originX = cubeX
#    originY = cubeY
#    originZ = cubeZ
#    myRoot.seedEdgeByNumber(number=seedCrackFront,
#                            edges=(myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
#                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
#                                                         cubeZ),))[0],))
#                                                         
#    crackEdge = myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
#                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
#                                                         cubeZ),))                                                     
#                                                         
    #    
    originX = cubeX - crack_depth
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = (degChanges)*math.pi/180*(0.5+i)
    	degLine = (degChanges)/180*math.pi*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)    
        
    
     # # # SEEDS FOR CRACK BOX
    degRad = degChanges/2*math.pi/180.0
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    

    degRad = 1*math.pi/180

    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=nrLines/8)        
    # # # SEEDS ALONG OUTER CIRCLE
    degRad = 3*degChanges*math.pi/180.0 
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=seedContain)        
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=seedContain)    
                            
                            
    innerDivider = innerCrack
    outerDivider = outerCrack
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
                            
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*innerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*innerDivider),))[0],))
    
    symEdgeX=myInst.edges.findAt(((cubeX,cubeY-notch_radius*math.sqrt(0.5),cubeZ-notch_radius*math.sqrt(0.5)),))[0]
    symEdgeZ=myInst.edges.findAt(((cubeX-notch_radius*math.sqrt(0.5),cubeY-notch_radius*math.sqrt(0.5),cubeZ),))[0]
    myRoot.seedEdgeByNumber(edges=(symEdgeX,),number=seedNotchEdge)     
    myRoot.seedEdgeByNumber(edges=(symEdgeZ,),number=seedNotchEdge)
    # # # CRACK FRONT
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    myRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    crackEdge = myInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))

                                                     
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size)                                                        
#

    
    
 



#   # # CREATING STEP    
    
    myModel.StaticStep(name='General', previous='Initial')
    
    # # # DEFINING CRACK  
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = cubeY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    #    
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    
    
    
    myRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=crackEdge), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.25, name='Crack-1', symmetric=ON)
    
    
    
    
    
    # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    
    myModel.ExpressionField(description='', 
                            expression='(Y-'+str(cubeY/2)+')/'+str(cubeY/2), localCsys=None, name='Bending Function')
#    mdb.models['Notch_Bot_E_D1_L13_ref'].loads['Loading'].setValues(
#    )
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=FIELD, 
                     field='Bending Function', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 
    
    
    
    
    
    # # # BOUNDARY CONDITIONS
    # underneath crack front
    underCrackLeft = myInst.faces.findAt(((cubeX-(crack_depth+crack_mesh_radius+crackBuffer+0.01),
                                           cubeY-0.01, cubeZ),))
    myModel.ZsymmBC(createStepName='General', localCsys=None, name='Sym_Z_Under_Crack_Left',
    			region=Region(faces=underCrackLeft))

    xSymFace = myInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    myModel.XsymmBC(createStepName='General', localCsys=None, name='Sym_X',
    			region=Region(faces=xSymFace))

#    bottomFace = myInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
#    myModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
#                           region=Region(faces=bottomFace),
#                            u2=0)
   
    # CRACK FRONT SYMMETRIES
    originX = cubeX   
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_L_'+str(i)
    	myModel.ZsymmBC(createStepName='General', localCsys=None, name=bcName,
    			region=Region(faces=myInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))

    


    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
		memory=90, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
		multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
    mdb.jobs[core_name].submit()
    mdb.jobs[core_name].waitForCompletion()
    
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    
    
    
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines
    global_seed_size = 0.15
    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
        
    
    
    
    

    cubeX_Sub = 5.0
    cubeY_Sub = 5.0
    cubeZ_Sub = 5.0
                
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    sub_name = 'Notch_Bot_E_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
    
    
    mySubModel = mdb.Model(name=sub_name, objectToCopy=
            mdb.models[core_name]) 

    mySubModel.setValues(description='Submodel notched cube',
                         globalJob=core_name+'.odb')  



    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    
    # CREATING CUBE 
    cubeSketchSub = mySubModel.ConstrainedSketch(name='cube_Sub', sheetSize=300)
    cubeSketchSub.rectangle(point1=(0,0),point2=(cubeX_Sub,cubeY_Sub))
    
    mySubModel.Part(dimensionality=THREE_D, name='Cube_Sub', type=DEFORMABLE_BODY)
    
    cubeSub = mySubModel.parts['Cube_Sub']
    cubeSub.BaseSolidExtrude(sketch=cubeSketchSub, depth=cubeZ_Sub)
    
    cutSketch = mySubModel.ConstrainedSketch(name='cube_cut_Sub', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = mySubModel.ConstrainedSketch(name='cut_raw_Sub', sheetSize=300,
                                       transform=cubeSub.MakeSketchTransform(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX_Sub,cubeY_Sub,cubeZ_Sub)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cubeSub.CutRevolve(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketchSub
    

    
    # # # CREATING CRACK SHELL
    crackSweep = mySubModel.ConstrainedSketch(name='crack_sweep_Sub', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = mySubModel.ConstrainedSketch(name='crack_shape_Sub', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    mySubModel.Part(dimensionality=THREE_D, name='Crack_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Crack_Sub'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep





    # # # # CREATING CRACK CIRCLE

    containShape = mySubModel.ConstrainedSketch(name='contain_shape_Sub', sheetSize=200.0) 
    innerCrack = crack_depth-crackBuffer-crack_mesh_radius
    outerCrack = crack_depth+crackBuffer+crack_mesh_radius
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    
    containShape.Line(point1=(0,outerCrack), point2=(0,-(outerCrack)))
    containShape.Line(point1=(0,outerCrack), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(innerCrack),0)),
                            point1=(-(innerCrack),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    mySubModel.Part(dimensionality=THREE_D, name='Contain_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Contain_Sub'].BaseShellRevolve(angle=360, sketch=
            containShape)




    # CUTTING
    centerX = cubeX-cubeX_Sub
    centerY = cubeY-cubeY_Sub
    centerZ = cubeZ-cubeZ_Sub
    centerCube = (centerX,centerY,centerZ)
    center = (cubeX,cubeY,cubeZ)
    
    
    
    mySubRoot = mySubModel.rootAssembly

    
    
    mySubRoot.Instance(name='Cube_Sub',part=mySubModel.parts['Cube_Sub'])
    mySubRoot.translate(instanceList=['Cube_Sub',], vector=centerCube)
    
    
    
    mySubRoot.Instance(name='Crack_Sub', part=mySubModel.parts['Crack_Sub'])
    mySubRoot.translate(instanceList=['Crack_Sub',], vector=center)

    mySubRoot.Instance(name='Contain_Sub', part=mySubModel.parts['Contain_Sub'])
    mySubRoot.translate(instanceList=['Contain_Sub',], vector=center)    
                              
    mySubInst = mySubRoot.InstanceFromBooleanCut(name='Cracked_Notch_Sub', 
                                  cuttingInstances=(mySubRoot.instances['Contain_Sub'],mySubRoot.instances['Crack_Sub'],),
                                  instanceToBeCut=mySubRoot.instances['Cube_Sub'], 
                                  originalInstances=SUPPRESS)    
    mySubPart = mySubModel.parts['Cracked_Notch_Sub']

    del mySubRoot.features['Cracked_Notch-1']
    del mySubModel.steps['General']        
    
    
    
     # # # #Datum Planes
    faceX = mySubPart.faces.findAt(((cubeX,-cubeY_Sub/2+cubeY,cubeZ-cubeZ_Sub/2),))[0]
    faceY = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub,cubeZ-cubeZ_Sub/2),))[0]
    faceZ = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub/2,cubeZ),))[0]
        

    z2 = mySubPart.DatumPlaneByOffset(plane=faceZ, offset=((crack_mesh_radius+crackBuffer)), flip=SIDE2)

    
    mySubPart.PartitionCellByDatumPlane(cells=mySubPart.cells[0:len(mySubPart.cells)], 
                                                        datumPlane=mySubPart.datums[z2.id]) 
                                                        
                                                        
    # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    mySubModel.Material(name='Steel')
    mySubModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    mySubModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    mySubPart.Set(cells=
        mySubPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    mySubPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        mySubPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    mySubRoot.makeIndependent(instances=(mySubInst,))
    
    
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # # CRACK FRONT
# #    seedCrackFront = 16
# #    originX = cubeX_Sub
# #    originY = cubeY_Sub
# #    originZ = cubeZ_Sub
# #    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
# #                            edges=(mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         cubeZ_Sub),))[0],))
# #                                                         
# #    crackEdge = mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
# #                                                         cubeZ_Sub),))                                                     
# #                                                         
    # #    
    originX = cubeX - crack_depth
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = (degChanges)*math.pi/180*(0.5+i)
    	degLine = (degChanges)/180*math.pi*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)    
        
    
    #SEEDS FOR CRACK BOX
    degRad = degChanges/2*math.pi/180.0
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    

    degRad = 1*math.pi/180

    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=nrLines/8)        
    # # SEEDS ALONG OUTER CIRCLE
    degRad = 3*degChanges*math.pi/180.0 
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_inner*math.cos(degRad),cubeY,cubeZ-contain_inner*math.sin(degRad)),))[0],),
                            number=seedContain)        
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-contain_outer*math.cos(degRad),cubeY,cubeZ-contain_outer*math.sin(degRad)),))[0],),
                            number=seedContain)    
                            
                            
    innerDivider = innerCrack
    outerDivider = outerCrack
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
                            
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*innerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*innerDivider),))[0],))
                            
        # # # # CRACK FRONT
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    crackEdge = mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))
    
                                                     
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch_Sub-1'],), size=global_seed_size)                                                        
    # #
    
    
    
     
    
    
    
    # #   # # CREATING STEP    
    
    mySubModel.StaticStep(name='General_Sub', previous='Initial')
    
    # # # # DEFINING CRACK  
    originX = cubeX 
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = cubeY_Sub-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    #    
    crackSurf = mySubInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+mySubInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    
    
    
    mySubRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=crackEdge), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.25, name='Crack-1', symmetric=ON)
    
    
    
    
    
    # # # GENERATING MESH
    mySubRoot.setElementType(regions=(mySubInst.cells[0:len(mySubInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    mySubRoot.generateMesh((mySubInst,))
    
    
    # # # LOADING   
    
    
    
    # # # # BOUNDARY CONDITIONS
    # # underneath crack front
    underCrackLeft = mySubInst.faces.findAt(((cubeX-(crack_depth+crack_mesh_radius+crackBuffer+0.01),
                                           cubeY-0.01, cubeZ),))
    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_Under_Crack_Left',
    			region=Region(faces=underCrackLeft))
    
    xSymFace = mySubInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    mySubModel.XsymmBC(createStepName='Initial', localCsys=None, name='Sym_X',
    			region=Region(faces=xSymFace))
    
    # #  bottomFace = mySubInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
    # #  mySubModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
    # #                           region=Region(faces=bottomFace),
    # #                            u2=0)
       
    # # CRACK FRONT SYMMETRIES
    originX = cubeX   
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_L_'+str(i)
    	mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
    			region=Region(faces=mySubInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))
    
    
    
    # # SUBMODEL BC's
    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
                  mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
                  mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),)
    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
                
    region = mySubRoot.sets['subDrivenBoundary']
    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
        region=region, globalStep='1', 
        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3),
        globalDrivingRegion='', 
        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)      
    
    
    
    
    
    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
    		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    		memory=90, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
    		multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
    		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
    		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
                
    mdb.jobs[sub_name].submit()
    mdb.jobs[sub_name].waitForCompletion()

cubeX = 20.0
cubeY = 20.0
cubeZ = 20.0
notch_radius = 1.0  
xBuffer = 0.5
yBuffer = 0.5


crack_array=np.arange(0.4,1.1,0.1)
#crack_array=np.array([0.4])
for l in range(0,len(crack_array)):
    far_away = 20.0
    short_far_away = 1.0
    crack_mesh_radius = 0.15
    nrInc = 10
    nrLines = 16
    degChanges = 360.0/nrLines*math.pi/180
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    crackBuffer=3*crack_mesh_radius/nrInc
    global_seed_size = 1.0
    seedCrackFront = 20
    seedNotchEdge = 25
    
    
    crack_depth = crack_array[l]
    core_name = 'Notch_Bot_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_ref'
    
    outerCrack = crack_depth+crack_mesh_radius+crackBuffer
    innerCrack = crack_depth-crack_mesh_radius-crackBuffer
    lineDist = 0.5
    
    myModel = mdb.Model(name=core_name)
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    cutSketch = myModel.ConstrainedSketch(name='cube_cut', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = myModel.ConstrainedSketch(name='cut_raw', sheetSize=300,
                                       transform=cube.MakeSketchTransform(sketchPlane=cube.faces.findAt(((cubeX/2,cubeY/2,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX,cubeY,cubeZ)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cube.CutRevolve(sketchPlane=cube.faces.findAt(((cubeX/2,cubeY/2,cubeZ),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cube.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketch
    
    
    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep
    
    
    
    
    # # # CREATING CRACK CIRCLE

    containShape = myModel.ConstrainedSketch(name='contain_shape', sheetSize=200.0) 

    containShape.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth-crackBuffer-crack_mesh_radius,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth+crackBuffer+crack_mesh_radius,0))
    
    containShape.Line(point1=(0,crack_depth+crackBuffer+crack_mesh_radius), point2=(0,-(crack_depth+crackBuffer+crack_mesh_radius)))
    containShape.Line(point1=(0,crack_depth+crackBuffer+crack_mesh_radius), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(crack_depth+crackBuffer+crack_mesh_radius),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(crack_depth-crackBuffer-crack_mesh_radius),0)),
                            point1=(-(crack_depth-crackBuffer-crack_mesh_radius),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    myModel.Part(dimensionality=THREE_D, name='Contain', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Contain'].BaseShellRevolve(angle=360, sketch=
            containShape)

    
    
    
    
                                      
     # # # ROOT, INSTANCE
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])
    
    center = (cubeX,cubeY-notch_radius,cubeZ)
    
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.translate(instanceList=['Crack',], vector=center)

    myRoot.Instance(name='Contain', part=myModel.parts['Contain'])
    myRoot.translate(instanceList=['Contain',], vector=center)    
                              
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Contain'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)    
    myPart = myModel.parts['Cracked_Notch']        
    
    
    myPart = myModel.parts['Cracked_Notch']
    
    
    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((cubeX/2,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((cubeX/2,cubeY/2,0),))[0]
        
    
    z1 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(cubeX-notch_radius)), flip=SIDE2)
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2)
    y2 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY-(cubeX-notch_radius)), flip=SIDE2)                                                            
    # # # FIND THE Y PLANE TO PARTITION
    r2 = notch_radius
    r1 = outerCrack
    
    yInt = ((math.pow(r1,2)-2*math.pow(r2,2))/(2*r2))
    rs = (math.pow(r1,2)-2*math.pow(r2,2))/(2*r2)
    xInt = math.sqrt(math.pow(r2,2)-math.pow(rs,2))
            
    y1 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY+yInt), flip=SIDE2)
    
                                                        
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                    datumPlane=myPart.datums[y1.id]) 
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[y2.id]) 

    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                    datumPlane=myPart.datums[z1.id]) 
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 

    
    
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
    
    
#    
#                                  
    myRoot.makeIndependent(instances=(myInst,))
#    
    # # # CRACK SURFACE MESHING
    originX = cubeX 
    originY = cubeY-(crack_depth+notch_radius)
    originZ = cubeZ
    lines = ()
    innerRegion = ()
    for i in range(0,nrLines/2):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        for j in range(0,nrInc):
            radius = crack_mesh_radius-crack_mesh_radius_increment*j
            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
            yLocCirc = radius*math.cos(degCirc)+originY
            zLocCirc = -radius*math.sin(degCirc)+originZ
            xLocCirc = cubeX
            
            yLocLine = (line_radius)*math.cos(degLine)+(originY)
            zLocLine = -(line_radius)*math.sin(degLine) + originZ
            xLocLine = cubeX
              
            yLocCell = line_radius*math.cos(degCirc)+(originY)
            zLocCell = -line_radius*math.sin(degCirc)+originZ
            xLocCell = cubeX
            
            if j == nrInc:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)            
            else:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,yLocCirc,zLocCirc),))[0],),number=1)    
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)    
                lines = lines + (myInst.edges.findAt(((originX,yLocLine,zLocLine),))[0],)
                innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,yLocCell,zLocCell),))[0],)

##    
##    # # # SETTING MESH CONTROLS    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX+0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX-0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
#                           regions=(myInst.cells.findAt(((cubeX/2,0.01,0.01),))[0],myInst.cells.findAt(((cubeX+0.01,0.01,0.01),))[0])
#                           , technique=SWEEP)
#    
#    
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
#                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+0.01,cubeY-0.01,0.01),))[0])
#                       , technique=SWEEP)
#                       
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX, 
#                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer/2)),))[0],)
#                       , technique=SWEEP)   
#      
#       
  
                            
    # # # SET MESH CONTROLS
                        
    
    # # #  UPPER SIDE SEED CRACK BOX
    degRad = 5.0/180*math.pi
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((originX,originY-outerCrack*math.cos(degRad),originZ-outerCrack*math.sin(degRad)),))[0],),
                            number=nrLines/4)    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((originX,originY-innerCrack*math.cos(degRad),originZ-innerCrack*math.sin(degRad)),))[0],),
                            number=nrLines/4)                                
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((originX,originY-(outerCrack+innerCrack)/2,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/2) 
                            
# # # BOTTOM SIDE SEED CRACK BOX
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((originX,originY-(outerCrack-crackBuffer/2),cubeZ),))[0],),
                            number=1) 
#myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((originX,originY-(outerCrack+crackBuffer/2),cubeZ),))[0],),
#                        number=1) 
    # # # SEEDS FOR CRACK FRONT
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    for i in range(0,(2*nrInc)+1):
        radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
        xLoc = -math.sqrt(1.0/2)*radius/10
        yLoc = originY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
        zLoc = cubeZ
        myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+xLoc,yLoc,zLoc),))[0],),
                                     number=seedCrackFront)
#    
#    # # # NOTCH SEEDS
#    outerDivider = outerCrack+notch_radius
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
#    myRoot.seedEdgeByNumber(number=seedNotchEdge,
#                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
#    
#    myRoot.seedEdgeByNumber(number=seedNotchEdge,
#                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
#    myRoot.seedEdgeByNumber(number=seedNotchEdge,
#                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
#   
#    # # # CRACK FRONT
#    
#    originX = cubeX + notch_radius
#    originY = cubeY
#    originZ = cubeZ
#    myRoot.seedEdgeByNumber(number=seedCrackFront,
#                            edges=(myInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
#                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
#                                                         cubeZ),))[0],))
#    # # # # 
    seedBottomBox = 4
    bottomEdges = (myInst.edges.findAt(((0.0,0.1*cubeY,cubeZ),))[0],myInst.edges.findAt(((cubeX,0.1*cubeY,cubeZ),))[0],
                   myInst.edges.findAt(((cubeX,0.1*cubeY,0.0),))[0],myInst.edges.findAt(((0.0,0.1*cubeY,0.0),))[0],)     
    myRoot.seedEdgeByNumber(number=seedBottomBox, edges=bottomEdges)
    #
#    

    myRoot.seedPartInstance(deviationFactor=
                                0.1, minSizeFactor=0.1,
                                regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
        
        
    # # # CREATE STEP
    myModel.StaticStep(name='General', previous='Initial')
#    
#    
#    
#    
    # # # DEFINING CRACK
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))        
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment    
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = originY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    x = crack_depth/2
    yCrackLoc = originY-(math.sqrt(math.pow(crack_depth,2)-math.pow(x,2)))
    xCrackLoc = originX-x
    #
    
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    myRoot.engineeringFeatures.ContourIntegral(
        collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
        crackNormal=((0.0, 0.0, 
        0.0), (0.0, 0.0, -1.0)),
        crackTip=Region(
        edges=myInst.edges.findAt(((xCrackLoc,yCrackLoc,originZ),))), extensionDirectionMethod=CRACK_NORMAL, 
        midNodePosition=0.5, name='Crack-1', symmetric=ON)
        
    
    

    
    # # # GENERATING MESH
        # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
    
    
    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    
    myModel.ExpressionField(description='', 
                            expression='(Y-'+str(cubeY/2)+')/'+str(cubeY/2), localCsys=None, name='Bending Function')
#    mdb.models['Notch_Bot_E_D1_L13_ref'].loads['Loading'].setValues(
#    )
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=FIELD, 
                     field='Bending Function', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 
    
    
    
    # # # BOUNDARY CONDITIONS
    # X SYMMETRY
    xSymFace = myInst.faces.getByBoundingBox(cubeX,-10,-10,cubeX,10,10)
    myModel.XsymmBC(createStepName='Initial', localCsys=None, name='Sym_X',
                region=Region(faces=xSymFace))

#    bottomFace = myInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
#    myModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
#                           region=Region(faces=bottomFace),
#                            u2=0)
    bcFaceUnder2 = myInst.faces.findAt(((0.01,cubeY-(cubeX-notch_radius)+0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder2',
            region=Region(faces=bcFaceUnder2)) 
      
    leftTopBcFace = myInst.faces.findAt(((0.01,cubeY-0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_TopLeft',
                region=Region(faces=leftTopBcFace))
    leftBotBcFace = myInst.faces.findAt(((0.01,0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_BotLeft',
                region=Region(faces=leftBotBcFace))

    
    # CRACK FRONT SYMMETRIES
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    for i in range(0,(nrInc)+1):
            radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
            xLoc = -radius/10
            yLoc = originY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
            bcName = 'Sym_Z_Crack_'+str(i)
            myModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
                    region=Region(faces=myInst.faces.findAt(((xLoc+originX,yLoc,originZ),))))
#    

    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
            memory=95, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
            multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
            SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
                
    mdb.jobs[core_name].submit()
    mdb.jobs[core_name].waitForCompletion()                
#                
#                
#                
#                
# 
#
    #
        
        
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    
    
    
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines*math.pi/180.0
    global_seed_size = 0.15
    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
        
    
    
    
    
    
    cubeX_Sub = 5.0
    cubeY_Sub = 5.0
    cubeZ_Sub = 5.0
                
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    sub_name = 'Notch_Bot_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
    
    
    mySubModel = mdb.Model(name=sub_name, objectToCopy=
            mdb.models[core_name]) 
    
    mySubModel.setValues(description='Submodel notched cube',
                         globalJob=core_name+'.odb')  
    
    
    
    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    
    # CREATING CUBE 
    cubeSketchSub = mySubModel.ConstrainedSketch(name='cube_Sub', sheetSize=300)
    cubeSketchSub.rectangle(point1=(0,0),point2=(cubeX_Sub,cubeY_Sub))
    
    mySubModel.Part(dimensionality=THREE_D, name='Cube_Sub', type=
        DEFORMABLE_BODY)
    
    cubeSub = mySubModel.parts['Cube_Sub']
    cubeSub.BaseSolidExtrude(sketch=cubeSketchSub, depth=cubeZ_Sub)
    
    cutSketch = mySubModel.ConstrainedSketch(name='cube_cut_Sub', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = mySubModel.ConstrainedSketch(name='cut_raw_Sub', sheetSize=300,
                                       transform=cubeSub.MakeSketchTransform(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX_Sub,cubeY_Sub,cubeZ_Sub)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cubeSub.CutRevolve(sketchPlane=cubeSub.faces.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cubeSub.edges.findAt(((cubeX_Sub,cubeY_Sub/2,cubeZ_Sub),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketchSub
    
    
    
    # # # CREATING CRACK SHELL
    crackSweep = mySubModel.ConstrainedSketch(name='crack_sweep_Sub', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = mySubModel.ConstrainedSketch(name='crack_shape_Sub', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    mySubModel.Part(dimensionality=THREE_D, name='Crack_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Crack_Sub'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep
    
    
    # # # CREATING CRACK CIRCLE
    
    containShape = mySubModel.ConstrainedSketch(name='contain_shape_Sub', sheetSize=200.0) 
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth-crackBuffer-crack_mesh_radius,0))
    
    containShape.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth+crackBuffer+crack_mesh_radius,0))
    
    containShape.Line(point1=(0,crack_depth+crackBuffer+crack_mesh_radius), point2=(0,-(crack_depth+crackBuffer+crack_mesh_radius)))
    containShape.Line(point1=(0,crack_depth+crackBuffer+crack_mesh_radius), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(crack_depth+crackBuffer+crack_mesh_radius),0)),
                            point1=(-(crack_depth+crackBuffer),0))
    
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(crack_depth-crackBuffer-crack_mesh_radius),0)),
                            point1=(-(crack_depth-crackBuffer-crack_mesh_radius),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    mySubModel.Part(dimensionality=THREE_D, name='Contain_Sub', type=
        DEFORMABLE_BODY)
    
    mySubModel.parts['Contain_Sub'].BaseShellRevolve(angle=360, sketch=
            containShape)
    
    
    # CUTTING
    centerX = cubeX-cubeX_Sub
    centerY = cubeY-cubeY_Sub
    centerZ = cubeZ-cubeZ_Sub
    centerCube = (centerX,centerY,centerZ)
    center = (cubeX,cubeY-notch_radius,cubeZ)
    
    
    
    mySubRoot = mySubModel.rootAssembly
    
    
    
    mySubRoot.Instance(name='Cube_Sub',part=mySubModel.parts['Cube_Sub'])
    mySubRoot.translate(instanceList=['Cube_Sub',], vector=centerCube)
    
    
    
    mySubRoot.Instance(name='Crack_Sub', part=mySubModel.parts['Crack_Sub'])
    mySubRoot.translate(instanceList=['Crack_Sub',], vector=center)
    
    mySubRoot.Instance(name='Contain_Sub', part=mySubModel.parts['Contain_Sub'])
    mySubRoot.translate(instanceList=['Contain_Sub',], vector=center)    
                              
    mySubInst = mySubRoot.InstanceFromBooleanCut(name='Cracked_Notch_Sub', 
                                  cuttingInstances=(mySubRoot.instances['Contain_Sub'],mySubRoot.instances['Crack_Sub'],),
                                  instanceToBeCut=mySubRoot.instances['Cube_Sub'], 
                                  originalInstances=SUPPRESS)    
    mySubPart = mySubModel.parts['Cracked_Notch_Sub']
    
    del mySubRoot.features['Cracked_Notch-1']
    del mySubModel.steps['General']        
    
    
    # Datum Planes
    faceX = mySubPart.faces.findAt(((cubeX,-cubeY_Sub/2+cubeY,cubeZ-cubeZ_Sub/2),))[0]
    faceY = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub,cubeZ-cubeZ_Sub/2),))[0]
    faceZ = mySubPart.faces.findAt(((cubeX-0.01,cubeY-cubeY_Sub/2,cubeZ),))[0]
        
                                                        
    
    
                                                        
    # # # FIND THE Y PLANE TO PARTITION
    r2 = notch_radius
    r1 = outerCrack
    
    yInt = ((math.pow(r1,2)-2*math.pow(r2,2))/(2*r2))
    rs = (math.pow(r1,2)-2*math.pow(r2,2))/(2*r2)
    xInt = math.sqrt(math.pow(r2,2)-math.pow(rs,2))
    
    
    z2 = mySubPart.DatumPlaneByOffset(plane=faceZ, offset=((crack_mesh_radius+crackBuffer)), flip=SIDE2)            
    y1 = mySubPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY_Sub+yInt), flip=SIDE2)
    
                                                        
    mySubPart.PartitionCellByDatumPlane(cells=mySubPart.cells[0:len(mySubPart.cells)], 
                                                    datumPlane=mySubPart.datums[y1.id]) 
    
    mySubPart.PartitionCellByDatumPlane(cells=mySubPart.cells[0:len(mySubPart.cells)], 
                                                        datumPlane=mySubPart.datums[z2.id]) 
    
    #-#    
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    mySubModel.Material(name='Steel')
    mySubModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    mySubModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    mySubPart.Set(cells=
        mySubPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    mySubPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        mySubPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    mySubRoot.makeIndependent(instances=(mySubInst,))
    
    
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # CRACK FRONT
    #    seedCrackFront = 16
    #    originX = cubeX_Sub
    #    originY = cubeY_Sub
    #    originZ = cubeZ_Sub
    #    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
    #                            edges=(mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
    #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
    #                                                         cubeZ_Sub),))[0],))
    #                                                         
    #    crackEdge = mySubInst.edges.findAt(((originX-math.sqrt(1.0/2.0)*crack_depth,
    #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
    #                                                         cubeZ_Sub),))                                                     
    #                                                         
    
    originX = cubeX 
    originY = cubeY-(crack_depth+notch_radius)
    originZ = cubeZ
    lines = ()
    innerRegion = ()
    for i in range(0,nrLines/2):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        for j in range(0,nrInc):
            radius = crack_mesh_radius-crack_mesh_radius_increment*j
            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.5+j) 
            yLocCirc = radius*math.cos(degCirc)+originY
            zLocCirc = -radius*math.sin(degCirc)+originZ
            xLocCirc = cubeX
            
            yLocLine = (line_radius)*math.cos(degLine)+(originY)
            zLocLine = -(line_radius)*math.sin(degLine) + originZ
            xLocLine = cubeX
              
            yLocCell = line_radius*math.cos(degCirc)+(originY)
            zLocCell = -line_radius*math.sin(degCirc)+originZ
            xLocCell = cubeX
            
            if j == nrInc:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)            
            else:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,yLocCirc,zLocCirc),))[0],),number=1)    
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)    
                lines = lines + (mySubInst.edges.findAt(((originX,yLocLine,zLocLine),))[0],)
                innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,yLocCell,zLocCell),))[0],)
        
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)    
        
    #-#
    # # #  UPPER SIDE SEED CRACK BOX
    degRad = 5.0/180*math.pi
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((originX,originY-outerCrack*math.cos(degRad),originZ-outerCrack*math.sin(degRad)),))[0],),
                            number=nrLines/4)    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((originX,originY-innerCrack*math.cos(degRad),originZ-innerCrack*math.sin(degRad)),))[0],),
                            number=nrLines/4)                                
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((originX,originY-(outerCrack+innerCrack)/2,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/2) 
                            
    # # # BOTTOM SIDE SEED CRACK BOX
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((originX,originY-(outerCrack-crackBuffer/2),cubeZ),))[0],),
                            number=1) 
    
    #-#
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    for i in range(0,(2*nrInc)+1):
        radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
        xLoc = -math.sqrt(1.0/2)*radius/10
        yLoc = originY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
        zLoc = cubeZ
        mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+xLoc,yLoc,zLoc),))[0],),
                                     number=seedCrackFront)                                                        
    #-#
    #    # # # NOTCH SEEDS
    #    outerDivider = outerCrack+notch_radius
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    #    myRoot.seedEdgeByNumber(number=seedNotchEdge,
    #                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    #    
    #    myRoot.seedEdgeByNumber(number=seedNotchEdge,
    #                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    #    myRoot.seedEdgeByNumber(number=seedNotchEdge,
    #                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    #   
    #    # # # CRACK FRONT
    #    
    #    originX = cubeX + notch_radius
    #    originY = cubeY
    #    originZ = cubeZ
    #    myRoot.seedEdgeByNumber(number=seedCrackFront,
    #                            edges=(myInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
    #                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
    #                                                         cubeZ),))[0],))
    #    # # # # 
    
    symEdgeX=myInst.edges.findAt(((0,cubeY-notch_radius*math.sqrt(0.5),cubeZ-notch_radius*math.sqrt(0.5)),))[0]
    symEdgeZ=myInst.edges.findAt(((cubeX-notch_radius*math.sqrt(0.5),cubeY-notch_radius*math.sqrt(0.5),0),))[0]
    myRoot.seedEdgeByNumber(edges=(symEdgeX,),number=seedNotchEdge)     
    myRoot.seedEdgeByNumber(edges=(symEdgeZ,),number=seedNotchEdge)
    
                                                     
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch_Sub-1'],), size=global_seed_size)                                                        
    #
    
    
    
     
    
    
    
    #   # # CREATING STEP    
    
    mySubModel.StaticStep(name='General_Sub', previous='Initial')
    # - #
    
    # # # DEFINING CRACK
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX-outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))        
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment    
    xInnerLoc = originX-innerContour_radius/2
    yInnerLoc = originY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    x = crack_depth/2
    yCrackLoc = originY-(math.sqrt(math.pow(crack_depth,2)-math.pow(x,2)))
    xCrackLoc = originX-x
    #
    
    crackSurf = mySubInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+mySubInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    mySubRoot.engineeringFeatures.ContourIntegral(
        collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
        crackNormal=((0.0, 0.0, 
        0.0), (0.0, 0.0, -1.0)),
        crackTip=Region(
        edges=mySubInst.edges.findAt(((xCrackLoc,yCrackLoc,originZ),))), extensionDirectionMethod=CRACK_NORMAL, 
        midNodePosition=0.5, name='Crack-1', symmetric=ON)
    
    
    # # # GENERATING MESH
    mySubRoot.setElementType(regions=(mySubInst.cells[0:len(mySubInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    mySubRoot.generateMesh((mySubInst,))
    
    
    # # # LOADING   
    
    
    
    # # # BOUNDARY CONDITIONS
    # underneath crack front
#    underCrackLeft = mySubInst.faces.findAt(((cubeX-(crack_depth+crack_mesh_radius+crackBuffer+0.01),
#                                           cubeY-0.01, cubeZ),))
#    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_Under_Crack_Left',
#                region=Region(faces=underCrackLeft))
    
    xSymFace = mySubInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    mySubModel.XsymmBC(createStepName='Initial', localCsys=None, name='Sym_X',
                region=Region(faces=xSymFace))
    
    #    bottomFace = mySubInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
    #    mySubModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
    #                           region=Region(faces=bottomFace),
    #                            u2=0)


#-#


    bcFaceUnder1 = mySubInst.faces.findAt(((cubeX-0.01,centerY+0.01,cubeZ),))
    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder1',
            region=Region(faces=bcFaceUnder1)) 
    
    bcFaceUnder2 = mySubInst.faces.findAt(((centerX+0.01,cubeY-0.01,cubeZ),))
    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder2',
            region=Region(faces=bcFaceUnder2))     
    # CRACK FRONT SYMMETRIES
    originX = cubeX
    originY = cubeY-notch_radius
    originZ = cubeZ
    for i in range(0,(nrInc)+1):
            radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
            xLoc = -radius/10
            yLoc = originY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
            bcName = 'Sym_Z_Crack_'+str(i)
            mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
                    region=Region(faces=mySubInst.faces.findAt(((xLoc+originX,yLoc,originZ),))))

    del mySubModel.boundaryConditions['Sym_Z_BotLeft']
    del mySubModel.boundaryConditions['Sym_Z_TopLeft']
#-#

    # SUBMODEL BC's
    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
                  mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
                  mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),)
    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
                
    region = mySubRoot.sets['subDrivenBoundary']
    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
        region=region, globalStep='1', 
        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3, 4, 5, 6),
        globalDrivingRegion='', 
        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)      
    
    
    
    
    
    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
            memory=90, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
            multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
            SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
                
    mdb.jobs[sub_name].submit()
    mdb.jobs[sub_name].waitForCompletion()                

cubeX = 20.0
cubeY = 20.0
cubeZ = 20.0
notch_radius = 1.0 
xBuffer = 15.0/30
yBuffer = 15.0/30

far_away = 20.0
short_far_away = 1.0
crack_mesh_radius = 0.15
nrInc = 10
nrLines = 32
degChanges = 360.0/nrLines*math.pi/180
crack_mesh_radius_increment = crack_mesh_radius/nrInc
crackBuffer=3*crack_mesh_radius/nrInc


#global_seed_size = 1.0


seedCrackFront = 20
seedNotchEdge = 25



crack_array=np.arange(0.4,1.1,0.1)
global_seed_size = 1.0
#crack_array = np.array([0.7])
for l in range(0,len(crack_array)):
    global_seed_size= 0.8
    crack_depth = crack_array[l]
    core_name = 'Notch_Sid_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_ref'
    
    
    outerCrack = crack_depth+crack_mesh_radius+crackBuffer
    innerCrack = crack_depth-crack_mesh_radius-crackBuffer
    lineDist = 0.5
    
    
    myModel = mdb.Model(name=core_name)
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX*2,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    cutSketch = myModel.ConstrainedSketch(name='cube_cut', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = myModel.ConstrainedSketch(name='cut_raw', sheetSize=300,
                                       transform=cube.MakeSketchTransform(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cube.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX,cubeY,cubeZ)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cube.CutRevolve(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cube.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketch
    
    
    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep
    
    
    
    
    
    
    # # # CREATING CRACK CONTAINER
    crackRoadSweep = myModel.ConstrainedSketch(name='road_sweep', sheetSize=far_away)
    crackRoadSweep.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    
    crackRoadShape = myModel.ConstrainedSketch(name='road_shape', sheetSize=far_away, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, notch_radius, 0.0, 0.0))
    
    # Crack Containers

    
    crackRoadShape.Line(point1=(0,0),point2=(-notch_radius,notch_radius))
    crackRoadShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    crackRoadShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    
    crackRoadShape.autoTrimCurve(curve1=crackRoadShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(outerCrack),0))
    crackRoadShape.autoTrimCurve(curve1=crackRoadShape.geometry.findAt((-innerCrack,0)),
                            point1=(-innerCrack,0))

    # Horizontal Dividers
    
    #scrackRoadShape.Line(point1=(-short_far_away,notch_radius+lineDist),
                        #point2=(far_away,notch_radius+lineDist))
    
    # Vertical Divider
    
    #crackRoadShape.Line(point1=(outerCrack+xBuffer,0),point2=(outerCrack+xBuffer,far_away))
    

    
    #crackRoadShape.Line(point1=(0,outerCrack),point2=(0,far_away))
    # Center Extension
    r2 = outerCrack
    x = -(math.pow(r2,2)/(2.0*notch_radius))
    y = math.sqrt(math.pow(r2,2) - math.pow(x,2))
    
    crackRoadShape.Line(point1=(x,y),point2=(x,y+far_away))
    
    myModel.Part(dimensionality=THREE_D, name='Crack_Road', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack_Road'].BaseShellSweep(path=crackRoadSweep, sketch=
        crackRoadShape)
    
    del crackRoadSweep, crackRoadShape
    
    
    
    
    
    
                                      
     # # # ROOT, INSTANCE
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])                                
    
    center = (cubeX,cubeY,cubeZ)
    crackCenter = (cubeX+notch_radius,cubeY,cubeZ)
    
    
    
    
    myRoot.Instance(name='Crack_Road',part=myModel.parts['Crack_Road'])
    myRoot.rotate(instanceList=['Crack_Road',], axisPoint=(0,0,0), axisDirection=(1,0,0), angle=90)
    myRoot.translate(instanceList=['Crack_Road',],vector=center)
    myRoot.rotate(instanceList=['Crack_Road',], axisPoint=center, axisDirection=(0,1,0), angle=-45)
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.translate(instanceList=['Crack',], vector=crackCenter)
    
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Crack_Road'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)
    
    
    
                              
    
    
    myPart = myModel.parts['Cracked_Notch']
    
    
    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((0.01,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((0.01,cubeY/2,0),))[0]
        
    x3 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX), flip=SIDE2)
    
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2)
    
    y2 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY-(cubeX-notch_radius)), flip=SIDE2) 
    
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[x3.id])                                                             
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[y2.id])     
    
    
    
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
    
    
    
                                  
    myRoot.makeIndependent(instances=(myInst,))
    
    # # # CRACK SURFACE MESHING
    originX = cubeX + crack_depth+notch_radius
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = degChanges*(0.5+i)
    	degLine = degChanges*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    
    # # # SETTING MESH CONTROLS    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX+0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX-0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
#                           regions=(myInst.cells.findAt(((cubeX/2,0.01,0.01),))[0],myInst.cells.findAt(((cubeX+0.01,0.01,0.01),))[0])
#                           , technique=SWEEP)
#    
#    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
                       regions=(myInst.cells.findAt(((cubeX*2-0.01,0.01,cubeZ-0.01),))[0],)
                       , technique=SWEEP)
#                       
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX, 
#                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer/2)),))[0],)
#                       , technique=SWEEP)   
      
       
       # # # SEEDS FOR CRACK BOX
    degRad = 1.0/180*math.pi #in order to target only sides of box
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    
      
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
                            
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
    
    # # # # BOTTOM BOX 
    seedBottomBox = 4
    bottomEdges = (myInst.edges.findAt(((0.0,0.1*cubeY,cubeZ),))[0],myInst.edges.findAt(((2*cubeX,0.1*cubeY,cubeZ),))[0],
                   myInst.edges.findAt(((2*cubeX,0.1*cubeY,0.0),))[0],myInst.edges.findAt(((0.0,0.1*cubeY,0.0),))[0],)     
    myRoot.seedEdgeByNumber(number=seedBottomBox, edges=bottomEdges)
    #
 
    # # # SET MESH CONTROLS
    # # #  LEFT SIDE SEED CRACK BOX
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-notch_radius-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX,cubeY,cubeZ-notch_radius-(outerCrack+innerCrack)/2),))[0],),
                            number=nrLines/4) 
    # # # RIGHT SIDE SEED CRACK BOX

    # # # SEEDS FOR CRACK FRONT
    originX = cubeX+notch_radius
    for i in range(0,(2*nrInc)+1):
         radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
         xLoc = radius/10
         yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
         myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLoc+originX,yLoc,cubeZ),))[0],),
                                 number=seedCrackFront)
    
    
    # # # NOTCH SEEDS
    outerDivider = outerCrack+notch_radius
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
   
    # # # CRACK FRONT
    
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    myRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(myInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
    
    
    # # # CREATE STEP
    myModel.StaticStep(name='General', previous='Initial')
    
    
    
    
    
    
    
    # # # DEFINING CRACK
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX+outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX+innerContour_radius/2
    yInnerLoc = cubeY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    x = crack_depth/2
    yCrackLoc = cubeY-(math.sqrt(math.pow(crack_depth,2)-math.pow(x,2)))
    xCrackLoc = x+cubeX+notch_radius
    #
    
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    myRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=myInst.edges.findAt(((xCrackLoc,yCrackLoc,originZ),))), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.5, name='Crack-1', symmetric=ON)
    
    
    
    
    
    
    
    # # # GENERATING MESH
        # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
    
    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    
    myModel.ExpressionField(description='', 
                            expression='(Y-'+str(cubeY/2)+')/'+str(cubeY/2), localCsys=None, name='Bending Function')
#    mdb.models['Notch_Bot_E_D1_L13_ref'].loads['Loading'].setValues(
#    )
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=FIELD, 
                     field='Bending Function', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 
    
    
    
    # # # BOUNDARY CONDITIONS
    
    #lowerBcFace = myInst.faces.getByBoundingBox(-1000,-1000,cubeZ,2*cubeX,cubeY-crack_depth-1,cubeZ)
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z',
    #			region=Region(faces=lowerBcFace))
    #
    #leftBcFace = myInst.faces.getByBoundingBox(-1000,-1000,cubeZ,cubeX,cubeY,cubeZ)
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_2',
    #			region=Region(faces=leftBcFace))
    
    #    xLoc = cubeX+notch_radius+xBuffer+outerCrack
    #    rightBcFace = myInst.faces.getByBoundingBox(xLoc,cubeY+1,cubeZ,1000,-100,cubeZ)
    #    print(rightBcFace)
    #    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_3',
    #    			region=Region(faces=rightBcFace))
    
    # underneath crack front
    # underCrackFace = myInst.faces.findAt(((cubeX+notch_radius,
                                           # cubeY-outerCrack-yBuffer, cubeZ),))
    # myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_Under_Crack_1',
    			# region=Region(faces=underCrackFace))
       
    
    # CRACK FRONT SYMMETRIES
    originX = cubeX+notch_radius
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_'+str(i)
    	myModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
    			region=Region(faces=myInst.faces.findAt(((xLoc+originX,yLoc,cubeZ),))))
    
		
    bcFaceUnder = myInst.faces.findAt(((cubeX+0.01,0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder',
			region=Region(faces=bcFaceUnder))
    bcFaceUnder2 = myInst.faces.findAt(((cubeX+0.01,cubeY-(cubeX-notch_radius)+0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder2',
    		region=Region(faces=bcFaceUnder2))  
			
    bottomEdge = (myInst.edges.findAt(((0.01,0,cubeZ),)),
                  myInst.edges.findAt(((cubeX-0.01,0,cubeZ),)),
                  myInst.edges.findAt(((cubeX+0.01,0,cubeZ),)),
                  myInst.edges.findAt(((2*cubeX-0.01,0,cubeZ),)),)
    myModel.DisplacementBC(name='Edge_Y', createStepName='Initial',
                           region=bottomEdge, u2=0)    
    myModel.DisplacementBC(name='Vertice_X', createStepName='Initial',
                           region=(myInst.vertices.findAt(((2*cubeX,0,cubeZ),)),))
    			
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder',
    		region=Region(faces=bcFaceUnder))    
    bcLeftSide = myInst.faces.getByBoundingBox(0,0,cubeZ,cubeX,cubeY,cubeZ)
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceLeft',
    		region=Region(faces=bcLeftSide))

    

#    bottomFace = myInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
#    myModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
#                           region=Region(faces=bottomFace),
#                            u2=0)


      
    ySides = np.array([1,cubeY-1])
    for i in range(0,len(ySides)):
        rightBcFace = myInst.faces.findAt(((2*cubeX-1,ySides[i],cubeZ),))
        bcName = 'rightBcFace'+str(i)
        myModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
        		region=Region(faces=rightBcFace))
    
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z',
    #			region=Region(faces=myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)))
    
    
    ##
    ##		
    ##
    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
    		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    		memory=95, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
    		multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
    		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
    		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
                
    mdb.jobs[core_name].submit()
    mdb.jobs[core_name].waitForCompletion()        
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    

    
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines*math.pi/180.0
    global_seed_size = 0.15
    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.15
    crackBuffer = 3*crack_mesh_radius/nrInc
    
    
    
    
    
    
    cubeX_Sub = 5.0
    cubeY_Sub = 5.0
    cubeZ_Sub = 5.0
    centerX = cubeX-cubeX_Sub
    centerY = cubeY-cubeY_Sub
    centerZ = cubeZ-cubeZ_Sub
    
    
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    sub_name = 'Notch_Bot_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
    
    
    mySubModel = mdb.Model(name=sub_name, objectToCopy=
        mdb.models[core_name]) 
    
    mySubModel.setValues(description='Submodel notched cube',
                     globalJob=core_name+'.odb')  
    
    mySubRoot = mySubModel.rootAssembly
    
    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    mySubPart = mySubModel.parts['Cracked_Notch']
    mySubInst = mySubRoot.instances['Cracked_Notch-1']
    mySubRoot.makeDependent(instances=(mySubInst,))
    # CREATING CUBE 
    
    
    cutSideSketch = mySubModel.ConstrainedSketch(name='cube_cut_side_Sub', sheetSize=100)
    cutSideSketch.rectangle(point1=(0,0),point2=(centerZ,-(cubeY+1.0)))
    
    
    
    cutRawS = mySubModel.ConstrainedSketch(name='cut_Side_Sub', sheetSize=300,
                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((2*cubeX,0.01,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,0.0),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(0,0,0)))
    cutRawS.retrieveSketch(sketch=cutSideSketch)
    
    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX*2,0.01,cubeZ/2),))[0],
                        sketchPlaneSide=SIDE1,
                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                        sketchOrientation=RIGHT,
                        sketch=cutRawS)
    
    
    cutFrontSketch = mySubModel.ConstrainedSketch(name='cube_cut_front_Sub', sheetSize=100)
    
    cutFrontSketch.Line(point1=(centerX,cubeY),point2=(0,cubeY))
    cutFrontSketch.Line(point1=(0,cubeY),point2=(0,0))
    cutFrontSketch.Line(point1=(0,0),point2=(2*cubeX,0))
    cutFrontSketch.Line(point1=(2*cubeX,0),point2=(2*cubeX,cubeY))
    cutFrontSketch.Line(point1=(2*cubeX,cubeY),point2=(2*cubeX-centerX,cubeY))
    cutFrontSketch.Line(point1=(2*cubeX-centerX,cubeY),point2=(2*cubeX-centerX,centerY))
    cutFrontSketch.Line(point1=(centerX,centerY),point2=(2*cubeX-centerX,centerY))
    cutFrontSketch.Line(point1=(centerX,centerY),point2=(centerX,cubeY))
    
    
    cutRaw = mySubModel.ConstrainedSketch(name='cut_front_Sub', sheetSize=300,
                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(0,0,0)))
    cutRaw.retrieveSketch(sketch=cutFrontSketch)
    
    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ),))[0],
                        sketchPlaneSide=SIDE1,
                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                        sketchOrientation=RIGHT,
                        sketch=cutRaw)
    
    mySubPart.regenerate()
    mySubRoot.makeIndependent(instances=(mySubInst,))
    
        # # # # # MESHING AGAIN. # # # # 
        
    
    originX = cubeX + crack_depth+notch_radius
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        for j in range(0,nrInc):
            radius = crack_mesh_radius-crack_mesh_radius_increment*j
            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
            xLocCirc = radius*math.cos(degCirc)+originX
            zLocCirc = -radius*math.sin(degCirc)+originZ
      
            xLocLine = (line_radius)*math.cos(degLine)+(originX)
            zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
            xLocCell = line_radius*math.cos(degCirc)+(originX)
            zLocCell = -line_radius*math.sin(degCirc)+originZ
    #        
            if j == nrInc:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)            
            else:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)    
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)    
                innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    
    # # # SETTING MESH CONTROLS    
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(mySubInst.cells.findAt(((cubeX+0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(mySubInst.cells.findAt(((cubeX-0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
                           regions=(myInst.cells.findAt(((cubeX*2-0.01,0.01,cubeZ-0.01),))[0],)
                           , technique=SWEEP)
    #                       
    #    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX, 
    #                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer/2)),))[0],)
    #                       , technique=SWEEP)   
      
       
       # # # SEEDS FOR CRACK BOX
    degRad = 1.0/180*math.pi #in order to target only sides of box
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    
      
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
                            
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
    
    # # # SET MESH CONTROLS
    # # #  LEFT SIDE SEED CRACK BOX
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-notch_radius-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX,cubeY,cubeZ-notch_radius-(outerCrack+innerCrack)/2),))[0],),
                            number=nrLines/4) 
    # # # RIGHT SIDE SEED CRACK BOX
    
    # # # SEEDS FOR CRACK FRONT
    originX = cubeX+notch_radius
    for i in range(0,(2*nrInc)+1):
         radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
         xLoc = radius/10
         yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
         myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLoc+originX,yLoc,cubeZ),))[0],),
                                 number=seedCrackFront)
    
    
    # # # NOTCH SEEDS
    outerDivider = outerCrack+notch_radius
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
       
    # # # CRACK FRONT
    
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(mySubInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
    
	
    
    mySubRoot.setElementType(regions=(mySubInst.cells[0:len(mySubInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    mySubRoot.generateMesh((mySubInst,))    
    
    
    
    
    
    del mySubModel.steps['General']
    mySubModel.StaticStep(name='General_Sub', previous='Initial')
    
    del mySubModel.boundaryConditions['bcFaceUnder']
    del mySubModel.boundaryConditions['rightBcFace0']   
    del mySubModel.boundaryConditions['Vertice_X']  
    del mySubModel.boundaryConditions['Edge_Y']           
    # # # BOUNDARY CONDITIONS                    
    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
              mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
              mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),
              mySubInst.faces.getByBoundingBox(2*cubeX-centerX,-1000,-1000,2*cubeX-centerX,1000,1000),)
    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
                
    region = mySubRoot.sets['subDrivenBoundary']
    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
        region=region, globalStep='1', 
        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3),
        globalDrivingRegion='', 
        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)  
        
#    symZFaces = (myInst.faces.findAt(((centerX+0.01,centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((2*cubeX-(centerX+0.01),centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((cubeX-0.01,centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((cubeX+0.01,centerY+0.01,cubeZ),)),)
    
#    mySubRoot.Set(faces=symZFaces, name='Z_Faces')
#    region=mySubRoot.sets['Z_Faces']
#    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceLeft_Sub',
#            region=region)    
            
        
    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
            memory=95, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
            multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
            SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)          
        
    mdb.jobs[sub_name].submit()
    mdb.jobs[sub_name].waitForCompletion() 

cubeX = 20.0
cubeY = 20.0
cubeZ = 20.0
notch_radius = 1.0 
xBuffer = 15.0/30
yBuffer = 15.0/30

far_away = 20.0
short_far_away = 1.0
crack_mesh_radius = 0.075
nrInc = 10
nrLines = 32
degChanges = 360.0/nrLines*math.pi/180
crack_mesh_radius_increment = crack_mesh_radius/nrInc
crackBuffer=2*crack_mesh_radius/nrInc


#global_seed_size = 1.0


seedCrackFront = 20
seedNotchEdge = 25



crack_array=np.array([0.2,0.3])
global_seed_size = 1.0
#crack_array = np.array([0.7])
for l in range(0,len(crack_array)):
    global_seed_size= 0.8
    crack_depth = crack_array[l]
    core_name = 'Notch_Sid_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_ref'
    
    
    outerCrack = crack_depth+crack_mesh_radius+crackBuffer
    innerCrack = crack_depth-crack_mesh_radius-crackBuffer
    lineDist = 0.5
    
    
    myModel = mdb.Model(name=core_name)
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX*2,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    cutSketch = myModel.ConstrainedSketch(name='cube_cut', sheetSize=100)
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    
    cutSketch.setAsConstruction(objectList=(
        cutSketch.geometry.findAt((0,0)),))
    
    cutSketch.Line(point1=(0,-notch_radius),point2=(0,notch_radius))
    cutSketch.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    cutSketch.autoTrimCurve(curve1=cutSketch.geometry.findAt((-notch_radius,0)),
                            point1=(-notch_radius,0))
    
    cutRaw = myModel.ConstrainedSketch(name='cut_raw', sheetSize=300,
                                       transform=cube.MakeSketchTransform(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=cube.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(cubeX,cubeY,cubeZ)))
    cutRaw.retrieveSketch(sketch=cutSketch)
    
    cube.CutRevolve(sketchPlane=cube.faces.findAt(((cubeX,cubeY/2,cubeZ),))[0],
                    sketchPlaneSide=SIDE1,
                    sketchUpEdge=cube.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                    sketchOrientation=RIGHT,
                    sketch = cutRaw,
                    angle =360)
                    
    del cutRaw, cutSketch, cubeSketch
    
    
    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.CircleByCenterPerimeter(center=(0,0), point1=(crack_depth,0))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep
    
    
    
    
    
    
    # # # CREATING CRACK CONTAINER
    crackRoadSweep = myModel.ConstrainedSketch(name='road_sweep', sheetSize=far_away)
    crackRoadSweep.CircleByCenterPerimeter(center=(0,0), point1=(notch_radius,0))
    
    crackRoadShape = myModel.ConstrainedSketch(name='road_shape', sheetSize=far_away, 
        transform=(1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, notch_radius, 0.0, 0.0))
    
    # Crack Containers

    
    crackRoadShape.Line(point1=(0,0),point2=(-notch_radius,notch_radius))
    crackRoadShape.CircleByCenterPerimeter(center=(0,0), point1=(outerCrack,0))
    crackRoadShape.CircleByCenterPerimeter(center=(0,0), point1=(innerCrack,0))
    
    
    crackRoadShape.autoTrimCurve(curve1=crackRoadShape.geometry.findAt((-(outerCrack),0)),
                            point1=(-(outerCrack),0))
    crackRoadShape.autoTrimCurve(curve1=crackRoadShape.geometry.findAt((-innerCrack,0)),
                            point1=(-innerCrack,0))

    # Horizontal Dividers
    
    #scrackRoadShape.Line(point1=(-short_far_away,notch_radius+lineDist),
                        #point2=(far_away,notch_radius+lineDist))
    
    # Vertical Divider
    
    #crackRoadShape.Line(point1=(outerCrack+xBuffer,0),point2=(outerCrack+xBuffer,far_away))
    

    
    #crackRoadShape.Line(point1=(0,outerCrack),point2=(0,far_away))
    # Center Extension
    r2 = outerCrack
    x = -(math.pow(r2,2)/(2.0*notch_radius))
    y = math.sqrt(math.pow(r2,2) - math.pow(x,2))
    
    crackRoadShape.Line(point1=(x,y),point2=(x,y+far_away))
    
    myModel.Part(dimensionality=THREE_D, name='Crack_Road', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack_Road'].BaseShellSweep(path=crackRoadSweep, sketch=
        crackRoadShape)
    
    del crackRoadSweep, crackRoadShape
    
    
    
    
    
    
                                      
     # # # ROOT, INSTANCE
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])                                
    
    center = (cubeX,cubeY,cubeZ)
    crackCenter = (cubeX+notch_radius,cubeY,cubeZ)
    
    
    
    
    myRoot.Instance(name='Crack_Road',part=myModel.parts['Crack_Road'])
    myRoot.rotate(instanceList=['Crack_Road',], axisPoint=(0,0,0), axisDirection=(1,0,0), angle=90)
    myRoot.translate(instanceList=['Crack_Road',],vector=center)
    myRoot.rotate(instanceList=['Crack_Road',], axisPoint=center, axisDirection=(0,1,0), angle=-45)
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.translate(instanceList=['Crack',], vector=crackCenter)
    
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Crack_Road'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)
    
    
    
                              
    
    
    myPart = myModel.parts['Cracked_Notch']
    
    
    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((0.01,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((0.01,cubeY/2,0),))[0]
        
    x3 = myPart.DatumPlaneByOffset(plane=faceX, offset=(cubeX), flip=SIDE2)
    
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2)
    
    y2 = myPart.DatumPlaneByOffset(plane=faceY, offset=(cubeY-(cubeX-notch_radius)), flip=SIDE2) 
    
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[x3.id])                                                             
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[y2.id])     
    
    
    
    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
    
    
    
                                  
    myRoot.makeIndependent(instances=(myInst,))
    
    # # # CRACK SURFACE MESHING
    originX = cubeX + crack_depth+notch_radius
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
    	degCirc = degChanges*(0.5+i)
    	degLine = degChanges*(1+i)	
    	for j in range(0,nrInc):
    		radius = crack_mesh_radius-crack_mesh_radius_increment*j
    		line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
    		xLocCirc = radius*math.cos(degCirc)+originX
    		zLocCirc = -radius*math.sin(degCirc)+originZ
      
    		xLocLine = (line_radius)*math.cos(degLine)+(originX)
    		zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
    		xLocCell = line_radius*math.cos(degCirc)+(originX)
    		zLocCell = -line_radius*math.sin(degCirc)+originZ
    #		
    		if j == nrInc:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)			
    		else:
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)	
    			myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)	
    			innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    
    # # # SETTING MESH CONTROLS    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX+0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((cubeX-0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
#                           regions=(myInst.cells.findAt(((cubeX/2,0.01,0.01),))[0],myInst.cells.findAt(((cubeX+0.01,0.01,0.01),))[0])
#                           , technique=SWEEP)
#    
#    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
                       regions=(myInst.cells.findAt(((cubeX*2-0.01,0.01,cubeZ-0.01),))[0],)
                       , technique=SWEEP)
#                       
#    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX, 
#                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer/2)),))[0],)
#                       , technique=SWEEP)   
      
       
       # # # SEEDS FOR CRACK BOX
    degRad = 1.0/180*math.pi #in order to target only sides of box
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    
      
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX+(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
                            
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
    
    # # # # BOTTOM BOX 
    seedBottomBox = 4
    bottomEdges = (myInst.edges.findAt(((0.0,0.1*cubeY,cubeZ),))[0],myInst.edges.findAt(((2*cubeX,0.1*cubeY,cubeZ),))[0],
                   myInst.edges.findAt(((2*cubeX,0.1*cubeY,0.0),))[0],myInst.edges.findAt(((0.0,0.1*cubeY,0.0),))[0],)     
    myRoot.seedEdgeByNumber(number=seedBottomBox, edges=bottomEdges)
    #
 
    # # # SET MESH CONTROLS
    # # #  LEFT SIDE SEED CRACK BOX
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-notch_radius-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX,cubeY,cubeZ-notch_radius-(outerCrack+innerCrack)/2),))[0],),
                            number=nrLines/4) 
    # # # RIGHT SIDE SEED CRACK BOX

    # # # SEEDS FOR CRACK FRONT
    originX = cubeX+notch_radius
    for i in range(0,(2*nrInc)+1):
         radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
         xLoc = radius/10
         yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
         myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLoc+originX,yLoc,cubeZ),))[0],),
                                 number=seedCrackFront)
    
    
    # # # NOTCH SEEDS
    outerDivider = outerCrack+notch_radius
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    myRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(myInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
   
    # # # CRACK FRONT
    
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    myRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(myInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
    
    
    # # # CREATE STEP
    myModel.StaticStep(name='General', previous='Initial')
    
    
    
    
    
    
    
    # # # DEFINING CRACK
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    
    outerContour_radius = crack_depth-0.5*crack_mesh_radius_increment
    xOuterLoc = originX+outerContour_radius/2
    yOuterLoc = originY-(math.sqrt(math.pow(outerContour_radius,2)-math.pow(outerContour_radius/2,2)))		
    
    #
    innerContour_radius = crack_depth+0.5*crack_mesh_radius_increment	
    xInnerLoc = originX+innerContour_radius/2
    yInnerLoc = cubeY-(math.sqrt(math.pow(innerContour_radius,2)-math.pow(innerContour_radius/2,2)))
    #
    x = crack_depth/2
    yCrackLoc = cubeY-(math.sqrt(math.pow(crack_depth,2)-math.pow(x,2)))
    xCrackLoc = x+cubeX+notch_radius
    #
    
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    myRoot.engineeringFeatures.ContourIntegral(
    	collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
    	crackNormal=((0.0, 0.0, 
    	0.0), (0.0, 0.0, -1.0)),
    	crackTip=Region(
    	edges=myInst.edges.findAt(((xCrackLoc,yCrackLoc,originZ),))), extensionDirectionMethod=CRACK_NORMAL, 
    	midNodePosition=0.5, name='Crack-1', symmetric=ON)
    
    
    
    
    
    
    
    # # # GENERATING MESH
        # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
    
    
    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    
    myModel.ExpressionField(description='', 
                            expression='(Y-'+str(cubeY/2)+')/'+str(cubeY/2), localCsys=None, name='Bending Function')
#    mdb.models['Notch_Bot_E_D1_L13_ref'].loads['Loading'].setValues(
#    )
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=FIELD, 
                     field='Bending Function', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 
    
    
    
    # # # BOUNDARY CONDITIONS
    
    #lowerBcFace = myInst.faces.getByBoundingBox(-1000,-1000,cubeZ,2*cubeX,cubeY-crack_depth-1,cubeZ)
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z',
    #			region=Region(faces=lowerBcFace))
    #
    #leftBcFace = myInst.faces.getByBoundingBox(-1000,-1000,cubeZ,cubeX,cubeY,cubeZ)
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_2',
    #			region=Region(faces=leftBcFace))
    
    #    xLoc = cubeX+notch_radius+xBuffer+outerCrack
    #    rightBcFace = myInst.faces.getByBoundingBox(xLoc,cubeY+1,cubeZ,1000,-100,cubeZ)
    #    print(rightBcFace)
    #    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_3',
    #    			region=Region(faces=rightBcFace))
    
    # underneath crack front
    # underCrackFace = myInst.faces.findAt(((cubeX+notch_radius,
                                           # cubeY-outerCrack-yBuffer, cubeZ),))
    # myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z_Under_Crack_1',
    			# region=Region(faces=underCrackFace))
       
    
    # CRACK FRONT SYMMETRIES
    originX = cubeX+notch_radius
    for i in range(0,(nrInc)+1):
    	radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
    	xLoc = radius/10
    	yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
    	bcName = 'Sym_Z_Crack_'+str(i)
    	myModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
    			region=Region(faces=myInst.faces.findAt(((xLoc+originX,yLoc,cubeZ),))))
    
		
    bcFaceUnder = myInst.faces.findAt(((cubeX+0.01,0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder',
			region=Region(faces=bcFaceUnder))
    bcFaceUnder2 = myInst.faces.findAt(((cubeX+0.01,cubeY-(cubeX-notch_radius)+0.01,cubeZ),))
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder2',
    		region=Region(faces=bcFaceUnder2))  
			
    bottomEdge = (myInst.edges.findAt(((0.01,0,cubeZ),)),
                  myInst.edges.findAt(((cubeX-0.01,0,cubeZ),)),
                  myInst.edges.findAt(((cubeX+0.01,0,cubeZ),)),
                  myInst.edges.findAt(((2*cubeX-0.01,0,cubeZ),)),)
    myModel.DisplacementBC(name='Edge_Y', createStepName='Initial',
                           region=bottomEdge, u2=0)    
    myModel.DisplacementBC(name='Vertice_X', createStepName='Initial',
                           region=(myInst.vertices.findAt(((2*cubeX,0,cubeZ),)),))
    			
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder',
    		region=Region(faces=bcFaceUnder))    
    bcLeftSide = myInst.faces.getByBoundingBox(0,0,cubeZ,cubeX,cubeY,cubeZ)
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceLeft',
    		region=Region(faces=bcLeftSide))

    

#    bottomFace = myInst.faces.getByBoundingBox(-10.0,0.0,-10.0,10.0,0.0,10.0)
#    myModel.DisplacementBC(name='Bottom_Y', createStepName='Initial', 
#                           region=Region(faces=bottomFace),
#                            u2=0)


      
    ySides = np.array([1,cubeY-1])
    for i in range(0,len(ySides)):
        rightBcFace = myInst.faces.findAt(((2*cubeX-1,ySides[i],cubeZ),))
        bcName = 'rightBcFace'+str(i)
        myModel.ZsymmBC(createStepName='Initial', localCsys=None, name=bcName,
        		region=Region(faces=rightBcFace))
    
    #myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='Sym_Z',
    #			region=Region(faces=myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)))
    
    
    ##
    ##		
    ##
    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
    		explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    		memory=95, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
    		multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
    		SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
    		, scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
#                
    mdb.jobs[core_name].submit()
    mdb.jobs[core_name].waitForCompletion()        
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
    

    
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines*math.pi/180.0
    global_seed_size = 0.15
    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.075
    crackBuffer = 2*crack_mesh_radius/nrInc
    
    
    
    
    
    
    cubeX_Sub = 5.0
    cubeY_Sub = 5.0
    cubeZ_Sub = 5.0
    centerX = cubeX-cubeX_Sub
    centerY = cubeY-cubeY_Sub
    centerZ = cubeZ-cubeZ_Sub
    
    
    crack_depth = crack_array[l]
    equalDiff = crack_depth-notch_radius
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    sub_name = 'Notch_Bot_D'+str(int(notch_radius))+'_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
    
    
    mySubModel = mdb.Model(name=sub_name, objectToCopy=
        mdb.models[core_name]) 
    
    mySubModel.setValues(description='Submodel notched cube',
                     globalJob=core_name+'.odb')  
    
    mySubRoot = mySubModel.rootAssembly
    
    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
    
    
    mySubPart = mySubModel.parts['Cracked_Notch']
    mySubInst = mySubRoot.instances['Cracked_Notch-1']
    mySubRoot.makeDependent(instances=(mySubInst,))
    # CREATING CUBE 
    
    
    cutSideSketch = mySubModel.ConstrainedSketch(name='cube_cut_side_Sub', sheetSize=100)
    cutSideSketch.rectangle(point1=(0,0),point2=(centerZ,-(cubeY+1.0)))
    
    
    
    cutRawS = mySubModel.ConstrainedSketch(name='cut_Side_Sub', sheetSize=300,
                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((2*cubeX,0.01,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,0.0),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(0,0,0)))
    cutRawS.retrieveSketch(sketch=cutSideSketch)
    
    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX*2,0.01,cubeZ/2),))[0],
                        sketchPlaneSide=SIDE1,
                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                        sketchOrientation=RIGHT,
                        sketch=cutRawS)
    
    
    cutFrontSketch = mySubModel.ConstrainedSketch(name='cube_cut_front_Sub', sheetSize=100)
    
    cutFrontSketch.Line(point1=(centerX,cubeY),point2=(0,cubeY))
    cutFrontSketch.Line(point1=(0,cubeY),point2=(0,0))
    cutFrontSketch.Line(point1=(0,0),point2=(2*cubeX,0))
    cutFrontSketch.Line(point1=(2*cubeX,0),point2=(2*cubeX,cubeY))
    cutFrontSketch.Line(point1=(2*cubeX,cubeY),point2=(2*cubeX-centerX,cubeY))
    cutFrontSketch.Line(point1=(2*cubeX-centerX,cubeY),point2=(2*cubeX-centerX,centerY))
    cutFrontSketch.Line(point1=(centerX,centerY),point2=(2*cubeX-centerX,centerY))
    cutFrontSketch.Line(point1=(centerX,centerY),point2=(centerX,cubeY))
    
    
    cutRaw = mySubModel.ConstrainedSketch(name='cut_front_Sub', sheetSize=300,
                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ),))[0],
                                        sketchPlaneSide=SIDE1,
                                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                                        sketchOrientation=RIGHT,
                                        origin=(0,0,0)))
    cutRaw.retrieveSketch(sketch=cutFrontSketch)
    
    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ),))[0],
                        sketchPlaneSide=SIDE1,
                        sketchUpEdge=mySubPart.edges.findAt(((2*cubeX,cubeY/2,cubeZ),))[0],
                        sketchOrientation=RIGHT,
                        sketch=cutRaw)
    
    mySubPart.regenerate()
    mySubRoot.makeIndependent(instances=(mySubInst,))
    
        # # # # # MESHING AGAIN. # # # # 
        
    
    originX = cubeX + crack_depth+notch_radius
    originY = cubeY
    originZ = cubeZ
    
    innerRegion = ()
    for i in range(0,nrLines/2):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        for j in range(0,nrInc):
            radius = crack_mesh_radius-crack_mesh_radius_increment*j
            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
            xLocCirc = radius*math.cos(degCirc)+originX
            zLocCirc = -radius*math.sin(degCirc)+originZ
      
            xLocLine = (line_radius)*math.cos(degLine)+(originX)
            zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
            xLocCell = line_radius*math.cos(degCirc)+(originX)
            zLocCell = -line_radius*math.sin(degCirc)+originZ
    #        
            if j == nrInc:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)            
            else:
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,cubeY,zLocCirc),))[0],),number=1)    
                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,cubeY,zLocLine),))[0],),number=1)    
                innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,cubeY,zLocCell),))[0],)
    
    
    # # # SETTING MESH CONTROLS    
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(mySubInst.cells.findAt(((cubeX+0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(mySubInst.cells.findAt(((cubeX-0.01,cubeY-notch_radius,cubeZ),))[0],), technique=SWEEP)
    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, 
                           regions=(myInst.cells.findAt(((cubeX*2-0.01,0.01,cubeZ-0.01),))[0],)
                           , technique=SWEEP)
    #                       
    #    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX, 
    #                       regions=(myInst.cells.findAt(((cubeX/2,cubeY-0.01,0.01),))[0],myInst.cells.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer/2)),))[0],)
    #                       , technique=SWEEP)   
      
       
       # # # SEEDS FOR CRACK BOX
    degRad = 1.0/180*math.pi #in order to target only sides of box
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+notch_radius+crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    
      
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX+(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
                            
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-(outerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(outerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)    
    
    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-(innerCrack+notch_radius)*math.cos(degRad),cubeY,cubeZ-(innerCrack+notch_radius)*math.sin(degRad)),))[0],),
                            number=nrLines/8)       
    
    # # # SET MESH CONTROLS
    # # #  LEFT SIDE SEED CRACK BOX
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-notch_radius-crack_depth,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                            number=nrLines/4)    
    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX,cubeY,cubeZ-notch_radius-(outerCrack+innerCrack)/2),))[0],),
                            number=nrLines/4) 
    # # # RIGHT SIDE SEED CRACK BOX
    
    # # # SEEDS FOR CRACK FRONT
    originX = cubeX+notch_radius
    for i in range(0,(2*nrInc)+1):
         radius = crack_depth+(crack_mesh_radius_increment*(nrInc-i))
         xLoc = radius/10
         yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
         myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLoc+originX,yLoc,cubeZ),))[0],),
                                 number=seedCrackFront)
    
    
    # # # NOTCH SEEDS
    outerDivider = outerCrack+notch_radius
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*notch_radius,cubeY,cubeZ-math.sqrt(1.0/2)*notch_radius),))[0],))
    
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX-math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
    mySubRoot.seedEdgeByNumber(number=seedNotchEdge,
                            edges=(mySubInst.edges.findAt(((cubeX+math.sqrt(1.0/2)*outerDivider,cubeY,cubeZ-math.sqrt(1.0/2)*outerDivider),))[0],))
       
    # # # CRACK FRONT
    
    originX = cubeX + notch_radius
    originY = cubeY
    originZ = cubeZ
    mySubRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(mySubInst.edges.findAt(((originX+math.sqrt(1.0/2.0)*crack_depth,
                                                         originY-math.sqrt(1.0/2.0)*crack_depth,
                                                         cubeZ),))[0],))
    
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
    
     
    
    
    mySubRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(mySubRoot.instances['Cracked_Notch-1'],), size=global_seed_size)    
    
    
    mySubRoot.setElementType(regions=(mySubInst.cells[0:len(mySubInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    mySubRoot.generateMesh((mySubInst,))    
    
    
    
    
    
    del mySubModel.steps['General']
    mySubModel.StaticStep(name='General_Sub', previous='Initial')
    
    del mySubModel.boundaryConditions['bcFaceUnder']
    del mySubModel.boundaryConditions['rightBcFace0']   
    del mySubModel.boundaryConditions['Vertice_X']  
    del mySubModel.boundaryConditions['Edge_Y']           
    # # # BOUNDARY CONDITIONS                    
    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
              mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
              mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),
              mySubInst.faces.getByBoundingBox(2*cubeX-centerX,-1000,-1000,2*cubeX-centerX,1000,1000),)
    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
                
    region = mySubRoot.sets['subDrivenBoundary']
    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
        region=region, globalStep='1', 
        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3),
        globalDrivingRegion='', 
        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)  
        
#    symZFaces = (myInst.faces.findAt(((centerX+0.01,centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((2*cubeX-(centerX+0.01),centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((cubeX-0.01,centerY+0.01,cubeZ),)),
#                 myInst.faces.findAt(((cubeX+0.01,centerY+0.01,cubeZ),)),)
    
#    mySubRoot.Set(faces=symZFaces, name='Z_Faces')
#    region=mySubRoot.sets['Z_Faces']
#    mySubModel.ZsymmBC(createStepName='Initial', localCsys=None, name='bcFaceLeft_Sub',
#            region=region)    
            
        
    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
            memory=95, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
            multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
            SINGLE, numCpus=8, numGPUs=0, numDomains=8, queue=None, resultsFormat=ODB
            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)          
        
    mdb.jobs[sub_name].submit()
    mdb.jobs[sub_name].waitForCompletion() 


