# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# crack_array - The range of crack lengths analyzed (linked to file names)
# odbSource - Location of ODB files
# saveDest - Destination to save the jIntegral maximum values as npy files
# saveDestcsv - Destination to save the csv files with max jIntegral and 
#               another file which indicates which picked set loc has max J
# core - Core name of the files that are looped (crack length in name varies)
# nrContour - Number of contours asked for in the analysis.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
import math
import numpy as np
import odbAccess as odb

jNotchValues = []
jMax = []
maxSetLoc = []
#crack_array = np.arange(2,11,1)
crack_array = np.array([4,5,6,7,8,9,10])
crackLength = crack_array
odbSource = 'D:/0_MASTER/ABAQUS/Notches/SubMod/Notch_B_Bot/ODB/'
saveDest = 'D:/0_MASTER/ABAQUS/Notches/SubMod/Notch_B_Bot/NPY/'
saveDestcsv = 'D:/0_MASTER/ABAQUS/Notches/SubMod/Notch_B_Bot/CSV/'
core = odbSource + 'Notch_Bot_D1_L'
nrContour = 10
notch_radius = 1.0


for j in range (0,len(crackLength)):
    if crackLength[j]<4:
        pickedSet = str(573)
    else:
        pickedSet = str(573)
    
    jobName = core+str(crackLength[j])+'_sub.odb'
    odbPath = jobName
    print(odbPath)
    od = odb.openOdb(path=odbPath)
    nrSubSet = (len(od.rootAssembly.nodeSets)-1)/nrContour    
    myStep = od.steps['General_Sub'] 
        
    for k in range(1,nrSubSet+1):
        # # # # Get J-INTEGRAL
        jIntLast = myStep.historyRegions['ElementSet . ALL ELEMENTS'].historyOutputs['J at CRACK_CRACK-1__PICKEDSET'+pickedSet+'-'+str(k)+'__Contour_10'].data[0][1]
        jNotchValues.append(jIntLast)

# # # # FINDING COORDINATES AND ANGLES OF EACH POINT ON CRACK LINE # # # # # #   
    myRoot=od.rootAssembly    
    labelList = []
    angleRad =  []

    
    originX = 20.0
    originY = 20.0-notch_radius
    crackLen = crackLength[j]/10.0
    for i in range(1,nrSubSet+1):
        coordList=[]
        nod = myRoot.nodeSets['CRACK_CRACK-1_J__PICKEDSET'+pickedSet+'-'+str(i)+'__Contour_01'].nodes[0]
        
        length = len(nod)
        radiusContour = []
        labelContour = []
        xSaver = []
        degChecker = []
        
        
        for i in range(0,length):
            trgNod = nod[i]
            trgCoord = nod[i].coordinates
            x = trgCoord[0]-originX
            y = trgCoord[1]-originY
            radius = math.sqrt(math.pow(x,2)+math.pow(y,2))
            
            if y>0:
                degChecker.append(1)
            else:
                degChecker.append(0)
            
            xSaver.append(x)
            radiusContour.append(radius)
            labelContour.append(nod[i].label)
        
        sortRadius = np.sort(radiusContour,)
        indexSort = np.argsort(radiusContour)
        index = int((len(sortRadius)-1)/2)
        centerLabel = labelContour[indexSort[index]]
        
        xArray = np.array(xSaver)
        xOrdered = xArray[indexSort]
        
        degCheckerArray = np.array(degChecker)
        degCheckerCenter = degChecker[indexSort[index]]
        xCenter = xOrdered[index]
        labelList.append(centerLabel)
        angle = math.asin(xCenter/crackLen)
    
        if degCheckerCenter==1:
            angle=-(math.pi+angle)
        angleRad.append(angle)
    
    angleDeg = np.array(angleRad)*180.0/math.pi
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


    od.close()            
    
    # Find the maximum J for the given crack length, jMax
    jFinal = np.array(jNotchValues)
    tempMax = max(jFinal)
    np.save(saveDest+'j_integral_bot_'+str(crackLength[j])+'_sub',jFinal)
    tempLoc = np.nonzero((jFinal==tempMax))[0][0]+1


    crackFace = np.concatenate(([abs(jFinal)],[angleDeg]),0)
    np.savetxt(saveDestcsv+'Bot_crack_length'+str(crackLength[j])+'_sub.csv',
               crackFace)  

    jNotchValues = []
    
    maxSetLoc.append(np.array(tempLoc))
    jMax.append(abs(tempMax))

# Save npy files that can later be converted to .mat    
np.save(saveDest+'j_max_Loc_bot_ALL_sub',maxSetLoc)
np.save(saveDest+'j_max_bot_ALL_sub',jMax)

# Save as CSV files, which can be opened by excel
np.savetxt(saveDestcsv+'j_max_Loc_bot_ALL_sub.csv',maxSetLoc,delimiter=',')
np.savetxt(saveDestcsv+'j_max_bot_ALL_sub.csv',jMax,delimiter=',')

