from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from abaqusConstants import *

import numpy as np
import math

# Dimensions 
cubeX = 50.0
cubeY = 50.0
cubeZ = 50.0

d = 76


# Mesh things
nrInc = 10
seedContain=30
seedCrackFront = 20
nrLines = 32
nrDivLin = nrLines/4
degChanges = 360.0/nrLines
global_seed_size = 0.8 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
seedNotchEdge = 20

xBuffer = 10/30.0
yBuffer = 10/30.0
crack_mesh_radius = 0.5
crackBuffer = 3*crack_mesh_radius/nrInc
radi_increment = crack_mesh_radius/nrInc


crack_array=np.arange(5.0,5.5,0.5)
#crack_array=np.array([5.0])
for l in range(0,len(crack_array)):
    
    
    def crackRatioFormula(depth):
        # INPUT IS NORMALIZED DEPTH, elliptical start
        formula = -5.0470*math.pow(depth,4)+14.2546*math.pow(depth,3)-11.7320*math.pow(depth,2) + 3.2707*depth + 0.4967
        return formula
    
    # Ratio = a/b    
    def ellipseParam(ratio,a):
        sec_coord = a/ratio*math.sqrt(1-math.pow(0.9,2))
        return sec_coord
        
    nrInc = 10
    seedContain=30
    seedCrackFront = 20
    nrLines = 32
    degChanges = 360.0/nrLines
    global_seed_size = 2.0 # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
    seedNotchEdge = 20
    
    xBuffer = 10/30.0
    yBuffer = 10/30.0
    crack_mesh_radius = 0.5
    crackBuffer = 3*crack_mesh_radius/nrInc
    radi_increment = crack_mesh_radius/nrInc
    
    
    crack_depth = crack_array[l]
    crackRatio = crackRatioFormula(crack_depth/d)  
    
    
    crack_mesh_radius_increment = crack_mesh_radius/nrInc
    core_name = 'Bot_E'+'_L'+str(int(round(crack_depth,1)*10))+'_'+'ref'
    
    myModel = mdb.Model(name=core_name)

    contain_inner1 = crack_depth-crackBuffer-crack_mesh_radius
    contain_outer1 = crack_depth+crackBuffer+crack_mesh_radius    
    contain_inner2 = crack_depth/crackRatio-crackBuffer-crack_mesh_radius
    contain_outer2 = crack_depth/crackRatio+crackBuffer+crack_mesh_radius      
    
    
    # CREATING CUBE 
    cubeSketch = myModel.ConstrainedSketch(name='cube', sheetSize=300)
    cubeSketch.rectangle(point1=(0,0),point2=(cubeX,cubeY))
    
    myModel.Part(dimensionality=THREE_D, name='Cube', type=
        DEFORMABLE_BODY)
    
    cube = myModel.parts['Cube']
    cube.BaseSolidExtrude(sketch=cubeSketch, depth=cubeZ)
    
    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.EllipseByCenterPerimeter(center=(0,0), axisPoint2=(crack_depth,0), axisPoint1=(0,crack_depth/crackRatio))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(-1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_depth/crackRatio, 0.0, 0.0))
    
    
              
    crackShape.Line(point1=(0,0), point2=(crack_mesh_radius,0))
    crackShape.radialPattern(centerPoint=(0,0), 
                             geomList=(crackShape.geometry.findAt((crack_mesh_radius/2,0)),),
                             number=nrLines, totalAngle=360.0, 
                             vertexList=())
    
    mesh_circles = np.arange(crack_mesh_radius_increment,
              crack_mesh_radius+crack_mesh_radius_increment,
              crack_mesh_radius_increment)                       
    for i in range(0,len(mesh_circles)):
        crackShape.CircleByCenterPerimeter(center=(0,0), point1=(mesh_circles[i],0))
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
    
    del crackShape, crackSweep





    # # # CREATING CRACK CIRCLE

    containShape = myModel.ConstrainedSketch(name='contain_shape', sheetSize=200.0) 
    innerCrack1 = crack_depth-crackBuffer-crack_mesh_radius
    innerCrack2 = crack_depth/crackRatio-crackBuffer-crack_mesh_radius
    outerCrack1 = crack_depth+crackBuffer+crack_mesh_radius
    outerCrack2 = crack_depth/crackRatio+crackBuffer+crack_mesh_radius
    containShape.EllipseByCenterPerimeter(center=(0,0), axisPoint2=(innerCrack1,0),axisPoint1=(0,innerCrack2))
    containShape.EllipseByCenterPerimeter(center=(0,0), axisPoint2=(outerCrack1,0),axisPoint1=(0,outerCrack2))
    
    containShape.Line(point1=(0,outerCrack2), point2=(0,-(outerCrack2)))
    containShape.Line(point1=(0,outerCrack2), point2=(0,200))
    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(outerCrack1),0)),
                            point1=(-(crack_depth+crackBuffer),0))

    containShape.autoTrimCurve(curve1=containShape.geometry.findAt((-(innerCrack1),0)),
                            point1=(-(innerCrack1),0))   
    containShape.setAsConstruction(objectList=(
        containShape.geometry.findAt((0,150)),))
    
    
    myModel.Part(dimensionality=THREE_D, name='Contain', type=
        DEFORMABLE_BODY)
    
    myModel.parts['Contain'].BaseShellRevolve(angle=360, sketch=
            containShape)




    # CUTTING
    myRoot = myModel.rootAssembly
    
    myRoot.Instance(name='Cube',part=myModel.parts['Cube'])
    
    center = (cubeX,cubeY,cubeZ)
    
    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.rotate(instanceList=['Crack'],axisPoint=(0,0,0), axisDirection=(0,0,1), angle=90)
    myRoot.translate(instanceList=['Crack',], vector=center)

    myRoot.Instance(name='Contain', part=myModel.parts['Contain'])
    myRoot.rotate(instanceList=['Contain'],axisPoint=(0,0,0), axisDirection=(0,0,1), angle=90)
    myRoot.translate(instanceList=['Contain',], vector=center)    
                              
    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                                  cuttingInstances=(myRoot.instances['Contain'],myRoot.instances['Crack'],),
                                  instanceToBeCut=myRoot.instances['Cube'], 
                                  originalInstances=SUPPRESS)    
    myPart = myModel.parts['Cracked_Notch']


    # Datum Planes
    faceX = myPart.faces.findAt(((0,cubeY/2,cubeZ/2),))[0]
    faceY = myPart.faces.findAt(((0.01,0,cubeZ/2),))[0]
    faceZ = myPart.faces.findAt(((0.01,cubeY/2,0),))[0]
    z2 = myPart.DatumPlaneByOffset(plane=faceZ, offset=(cubeZ-(crack_mesh_radius+crackBuffer)), flip=SIDE2) 
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[z2.id]) 

    # # # MATERIAL MODEL
    Elasticity = 210000
    v = 0.3
    myModel.Material(name='Steel')
    myModel.materials['Steel'].Elastic(table=((Elasticity, v), ))
    myModel.HomogeneousSolidSection(material='Steel', name='plate', 
        thickness=None)
    
    myPart.Set(cells=
        myPart.cells.getByBoundingBox(-1000.0,-1000.0,-1000,1000.0
            ,1000.0,1000.0), name='Set-1')    
        
    myPart.SectionAssignment(offset=0.0, 
        offsetField='', offsetType=MIDDLE_SURFACE, region=
        myPart.sets['Set-1'], sectionName='plate', 
        thicknessAssignment=FROM_SECTION) 
                                      
    myRoot.makeIndependent(instances=(myInst,))
    
 
    
    
    
    
    
    # # # MESHING OMG 
    
    # # # CRACK FRONT
##    originX = cubeX
##    originY = cubeY
##    originZ = cubeZ

    #   # # CREATING STEP    
    
    myModel.StaticStep(name='General', previous='Initial')

# # # DEFINING CRACK      
    outerContour_radius = crack_depth-0.5*radi_increment
    xOuterLoc = cubeX-0.001
    yOuterLoc = cubeY-crack_depth+0.005       
    
    #
    innerContour_radius = crack_depth+0.5*radi_increment    
    xInnerLoc = cubeX-0.001
    yInnerLoc = cubeY-crack_depth-0.005
    #
    yCrackLoc = cubeY-crack_depth*0.9
    xCrackLoc = cubeX-ellipseParam(crackRatio,crack_depth)
        #
            
    crackSurf = myInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+myInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
    crackEdge = myInst.edges.findAt(((xCrackLoc,yCrackLoc,cubeZ),))
    myRoot.engineeringFeatures.ContourIntegral(
        collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
        crackNormal=((0.0, 0.0, 
        0.0), (0.0, 0.0, 1.0)),
        crackTip=Region(
        edges=crackEdge,), extensionDirectionMethod=CRACK_NORMAL, 
        midNodePosition=0.25, name='Crack-1', symmetric=ON)    
        
    myRoot.seedEdgeByNumber(edges=crackEdge, number=10)
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
                                elemCode=C3D20, elemLibrary=STANDARD),))    


#    
    # # # LOADING
    loadingFace = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    myRoot.Surface(side1Faces=(loadingFace,), name='loading_surface')
    myModel.Pressure(amplitude=UNSET, distributionType=UNIFORM, field='', 
                     name='Loading', createStepName='General', magnitude=-100,
                     region=myRoot.surfaces['loading_surface']) 


    seedCrackFront = 16
##    originX = cubeX
##    originY = cubeY
##    originZ = cubeZ
    myRoot.seedEdgeByNumber(number=seedCrackFront,
                            edges=(crackEdge[0],))

    originX = cubeX
    originY = cubeY -crack_depth
    originZ = cubeZ
    
    xLocCirc = cubeX
    xLocLine = cubeX
    xLocCell = cubeX    
    
    
    innerFaces = ()
    innerRegion = ()
    for i in range(0,nrLines/2):
        degCirc = (degChanges)*math.pi/180*(0.5+i)
        degLine = (degChanges)/180*math.pi*(1+i)    
        for j in range(0,nrInc):
            radius = crack_mesh_radius-crack_mesh_radius_increment*j
            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
            yLocCirc = radius*math.cos(degCirc)+originY
            zLocCirc = -radius*math.sin(degCirc)+originZ
      
            yLocLine = (line_radius)*math.cos(degLine)+(originY)
            zLocLine = -(line_radius)*math.sin(degLine) + originZ
      
            yLocCell = line_radius*math.cos(degCirc)+(originY)
            zLocCell = -line_radius*math.sin(degCirc)+originZ
    #        
            if j == nrInc:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX,yLocLine,zLocLine),))[0],),number=1)            
            else:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,yLocCirc,zLocCirc),))[0],),number=1)    
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)    
                innerRegion = innerRegion + (myInst.cells.findAt(((xLocCell,yLocCell,zLocCell),))[0],)
                innerFaces = innerFaces + (myInst.faces.findAt(((xLocCell,yLocCell,zLocCell),))[0],)
    
    myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)                                                  
    #    originX = cubeX - crack_depth

    def ellipsePos(a,b,x):
        y = b*math.sqrt(math.pow(1,2)-math.pow((x/a),2))
        return y
        
    degRad = degChanges/2*math.pi/180.0
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-crack_depth/crackRatio,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
                        number=nrLines/4)    
    
    degRad = 1*math.pi/180
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-ellipsePos(contain_inner1,contain_inner2,0.05),cubeY,cubeZ-0.05),))[0],),
                        number=nrLines/8)    
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-ellipsePos(contain_outer1,contain_outer2,0.05),cubeY,cubeZ-0.05),))[0],),
                        number=nrLines/8)      
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-ellipsePos(contain_inner1,contain_inner2,0.8),cubeY,cubeZ-0.8),))[0],),
                        number=10)    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX,cubeY-contain_inner1*math.sin(pi/4),cubeZ-math.sin(pi/4)*contain_inner1),))[0],),
                        number=10)    
            
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-ellipsePos(contain_outer1,contain_outer2,0.8),cubeY,cubeZ-0.8),))[0],),
                        number=10)  
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX-0.01,cubeY,cubeZ),))[0],),
                        number=10)  
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((cubeX,cubeY-0.01,cubeZ),))[0],),
                        number=10)  
      
#                                                     
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size)                                                        
#

    #    
    
    
    # # # GENERATING MESH
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
        elemCode=C3D20, elemLibrary=STANDARD),))
    myRoot.generateMesh((myInst,))
    
#    
# 
#
#
#

    
#    
#    
#    
#    
    # # # BOUNDARY CONDITIONS
    # underneath crack front
    underCrackLeft = myInst.faces.findAt(((cubeX-0.01,
                                           cubeY-(contain_outer2+0.05), cubeZ),))
    myModel.ZsymmBC(createStepName='General', localCsys=None, name='Sym_Z_Under_Crack_Left',
                region=Region(faces=underCrackLeft))
    
    xSymFace = myInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
    myModel.XsymmBC(createStepName='General', localCsys=None, name='Sym_X_Main',
                region=Region(faces=xSymFace))
    
    myModel.XsymmBC(createStepName='General', localCsys=None, name='Sym_X_CrackSide',
                region=innerFaces)
            
    # CRACK FRONT SYMMETRIES
    originX = cubeX   
    for i in range(0,(nrInc)+1):
            radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
            xLoc = radius/10
            yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
            bcName = 'Sym_Z_Crack_L_'+str(i)
            myModel.ZsymmBC(createStepName='General', localCsys=None, name=bcName,
                    region=Region(faces=myInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))

    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
                        
                
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
        explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
        memory=90, memoryUnits=PERCENTAGE, model=core_name, modelPrint=OFF, 
        multiprocessingMode=DEFAULT, name=core_name, nodalOutputPrecision=
        SINGLE, numCPUs=4, numGPUs=4, numDomains=4, queue=None, resultsFormat=ODB
        , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)                
#
#    mdb.jobs[core_name].submit()
#    mdb.jobs[core_name].waitForCompletion()
#    
    
#    
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #                 
## # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
#    
#    
#
#    
#    nrInc = 10
#    seedContain=30
#    seedCrackFront = 20
#    nrLines = 32
#    # degChanges = 360.0/nrLines*math.pi/180.0
#    degChanges = 360.0/nrLines
#    global_seed_size = 0.3
#    # GOOD FOR 1.8, 0.20 ACCEPTED BY PROGRAM
#    seedNotchEdge = 20
#    
#    xBuffer = 10/30.0
#    yBuffer = 10/30.0
#    crack_mesh_radius = 0.5
#    crackBuffer = 3*crack_mesh_radius/nrInc
#    radi_increment = crack_mesh_radius/nrInc
#    
#    
#    
#    
#    
#    cubeX_Sub = 10.0
#    cubeY_Sub = 10.0
#    cubeZ_Sub = 10.0
#    centerX = cubeX-cubeX_Sub
#    centerY = cubeY-cubeY_Sub
#    centerZ = cubeZ-cubeZ_Sub
#    
#    
#    crack_depth = crack_array[l]
#    equalDiff = crack_depth
#    
#    crack_mesh_radius_increment = crack_mesh_radius/nrInc
#    sub_name = 'Bot_E_L'+str(int(round(crack_depth,1)*10))+'_'+'sub'
#    
#    
#    mySubModel = mdb.Model(name=sub_name, objectToCopy=
#        mdb.models[core_name]) 
#    
#    mySubModel.setValues(description='Submodel notched cube',
#                     globalJob=core_name+'.odb')  
#    
#    mySubRoot = mySubModel.rootAssembly
#    
#    contain_inner = crack_depth-crackBuffer-crack_mesh_radius
#    contain_outer = crack_depth+crackBuffer+crack_mesh_radius    
#    
#    
#    mySubPart = mySubModel.parts['Cracked_Notch']
#    mySubInst = mySubRoot.instances['Cracked_Notch-1']
#    mySubRoot.makeDependent(instances=(mySubInst,))
#    # CREATING CUBE 
#    
#    
#    cutSideSketch = mySubModel.ConstrainedSketch(name='cube_cut_side_Sub', sheetSize=100)
#    cutSideSketch.rectangle(point1=(0,0),point2=(centerZ,-(cubeY+1.0)))
#    
#        
#        
#    cutRawS = mySubModel.ConstrainedSketch(name='cut_Sub', sheetSize=300,
#                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,0.01),))[0],
#                                        sketchPlaneSide=SIDE1,
#                                        sketchUpEdge=mySubPart.edges.findAt(((cubeX,cubeY/2,0.0),))[0],
#                                        sketchOrientation=RIGHT,
#                                        origin=(0,0,0)))
#    cutRawS.retrieveSketch(sketch=cutSideSketch)
#        
#    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ/2),))[0],
#                        sketchPlaneSide=SIDE1,
#                        sketchUpEdge=mySubPart.edges.findAt(((cubeX,cubeY/2,0),))[0],
#                        sketchOrientation=RIGHT,
#                        sketch=cutRawS)
#        
#        
#    cutFrontSketch = mySubModel.ConstrainedSketch(name='cube_cut_front_Sub', sheetSize=100)
#    
#    cutFrontSketch.Line(point1=(centerX,cubeY),point2=(0,cubeY))
#    cutFrontSketch.Line(point1=(0,cubeY),point2=(0,0))
#    cutFrontSketch.Line(point1=(0,0),point2=(2*cubeX,0))
#    cutFrontSketch.Line(point1=(2*cubeX,0),point2=(2*cubeX,cubeY))
#    cutFrontSketch.Line(point1=(2*cubeX,cubeY),point2=(2*cubeX-centerX,cubeY))
#    cutFrontSketch.Line(point1=(2*cubeX-centerX,cubeY),point2=(2*cubeX-centerX,centerY))
#    cutFrontSketch.Line(point1=(centerX,centerY),point2=(2*cubeX-centerX,centerY))
#    cutFrontSketch.Line(point1=(centerX,centerY),point2=(centerX,cubeY))
#        
#        
#    cutRaw = mySubModel.ConstrainedSketch(name='cut_front_Sub', sheetSize=300,
#                                       transform=mySubPart.MakeSketchTransform(sketchPlane=mySubPart.faces.findAt(((cubeX/2,0.01,cubeZ),))[0],
#                                        sketchPlaneSide=SIDE1,
#                                        sketchUpEdge=mySubPart.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
#                                        sketchOrientation=RIGHT,
#                                        origin=(0,0,0)))
#    cutRaw.retrieveSketch(sketch=cutFrontSketch)
#    
#    mySubPart.CutExtrude(sketchPlane=mySubPart.faces.findAt(((cubeX,0.01,cubeZ),))[0],
#                        sketchPlaneSide=SIDE1,
#                        sketchUpEdge=mySubPart.edges.findAt(((cubeX,cubeY/2,cubeZ),))[0],
#                        sketchOrientation=RIGHT,
#                        sketch=cutRaw)
#        
#    mySubPart.regenerate()
#    mySubRoot.makeIndependent(instances=(mySubInst,))  
#    
#    
#    
#    
#    del mySubModel.steps['General']
#    mySubModel.StaticStep(name='General_Sub', previous='Initial')
#    
#    drivenFaces = (mySubInst.faces.getByBoundingBox(centerX,-1000,-1000,centerX,1000,1000),
#          mySubInst.faces.getByBoundingBox(-1000,centerY,-1000,1000,centerY,1000),  
#          mySubInst.faces.getByBoundingBox(-1000,-1000,centerZ,1000,1000,centerZ),
#          mySubInst.faces.getByBoundingBox(cubeX-centerX,-1000,-1000,cubeX-centerX,1000,1000),)
#    
#    mySubRoot.Set(faces=drivenFaces, name='subDrivenBoundary')
#                
#    region = mySubRoot.sets['subDrivenBoundary']
#    mySubModel.SubmodelBC(name='subBC', createStepName='General_Sub',
#        region=region, globalStep='1', 
#        globalIncrement=0, timeScale=OFF, dof=(1, 2, 3),
#        globalDrivingRegion='', 
#        absoluteExteriorTolerance=0.0, exteriorTolerance=0.05)      
##    
#    # # # CRACK FRONT
###    originX = cubeX
#    originX = cubeX
#    originY = cubeY - crack_depth
#    originZ = cubeZ
#    
#    xLocCirc = cubeX
#    xLocLine = cubeX
#    xLocCell = cubeX  
#    
#    innerFaces = ()
#    innerRegion = ()
#    for i in range(0,nrLines/2):
#        degCirc = (degChanges)*math.pi/180*(0.5+i)
#        degLine = (degChanges)/180*math.pi*(1+i)    
#        for j in range(0,nrInc):
#            radius = crack_mesh_radius-crack_mesh_radius_increment*j
#            line_radius = crack_mesh_radius - crack_mesh_radius_increment*(0.01+j) 
#            yLocCirc = radius*math.cos(degCirc)+originY
#            zLocCirc = -radius*math.sin(degCirc)+originZ
#      
#            yLocLine = (line_radius)*math.cos(degLine)+(originY)
#            zLocLine = -(line_radius)*math.sin(degLine) + originZ
#      
#            yLocCell = line_radius*math.cos(degCirc)+(originY)
#            zLocCell = -line_radius*math.sin(degCirc)+originZ
#    #        
#            if j == nrInc:
#                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX,yLocLine,zLocLine),))[0],),number=1)            
#            else:
#                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocCirc,yLocCirc,zLocCirc),))[0],),number=1)    
#                mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((xLocLine,yLocLine,zLocLine),))[0],),number=1)    
#                innerRegion = innerRegion + (mySubInst.cells.findAt(((xLocCell,yLocCell,zLocCell),))[0],)
#                innerFaces = innerFaces + (mySubInst.faces.findAt(((xLocCell,yLocCell,zLocCell),))[0],)
#    
#    mySubRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=innerRegion, technique=SWEEP)                                                  
#    #    originX = cubeX - crack_depth
#
#    def ellipsePos(a,b,x):
#        y = b*math.sqrt(math.pow(1,2)-math.pow((x/a),2))
#        return y
#    
#    degRad = degChanges/2*math.pi/180.0
#    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-crack_depth/crackRatio,cubeY,cubeZ-(crack_mesh_radius+crackBuffer)),))[0],),
#                        number=nrLines/4)    
#    
#    degRad = 1*math.pi/180
#    
#    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-ellipsePos(contain_inner1,contain_inner2,0.05),cubeY,cubeZ-0.05),))[0],),
#                        number=nrLines/8)    
#    
#    mySubRoot.seedEdgeByNumber(edges=(mySubInst.edges.findAt(((cubeX-ellipsePos(contain_outer1,contain_outer2,0.05),cubeY,cubeZ-0.05),))[0],),
#                        number=nrLines/8)  
#   
##                                                     
#    mySubRoot.seedPartInstance(deviationFactor=
#                            0.1, minSizeFactor=0.1,
#                            regions=(mySubRoot.instances['Cracked_Notch-1'],), size=global_seed_size)                                                        
##
#
#    #    
## # # DEFINING CRACK      
#    outerContour_radius = crack_depth-0.5*radi_increment
#    xOuterLoc = cubeX-0.001
#    yOuterLoc = cubeY-crack_depth+0.005       
#    
#    #
#    innerContour_radius = crack_depth+0.5*radi_increment    
#    xInnerLoc = cubeX-0.001
#    yInnerLoc = cubeY-crack_depth-0.005
#    #
#    yCrackLoc = cubeY-crack_depth*0.9
#    xCrackLoc = cubeX-ellipseParam(crackRatio,crack_depth)
#        #
#            
#    crackSurf = mySubInst.faces.findAt(((xOuterLoc,yOuterLoc,cubeZ),))+mySubInst.faces.findAt(((xInnerLoc,yInnerLoc,cubeZ),))
#    crackEdge = mySubInst.edges.findAt(((xCrackLoc,yCrackLoc,cubeZ),))
#    mySubRoot.engineeringFeatures.ContourIntegral(
#        collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
#        crackNormal=((0.0, 0.0, 
#        0.0), (0.0, 0.0, 1.0)),
#        crackTip=Region(
#        edges=crackEdge,), extensionDirectionMethod=CRACK_NORMAL, 
#        midNodePosition=0.25, name='Crack-1', symmetric=ON)    
#        
#    mySubRoot.seedEdgeByNumber(edges=crackEdge, number=10)
#    mySubRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
#                                elemCode=C3D20, elemLibrary=STANDARD),))    
#
#        
#    
#    # # # GENERATING MESH
#    mySubRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
#        elemCode=C3D20, elemLibrary=STANDARD),))
#
#    mySubRoot.generateMesh((myInst,))
#    
#
#    
#    
#    underCrackLeft = mySubInst.faces.findAt(((cubeX-0.01,
#                                           cubeY-(contain_outer2+0.05), cubeZ),))
#    mySubModel.ZsymmBC(createStepName='General_Sub', localCsys=None, name='Sym_Z_Under_Crack_Left',
#                region=Region(faces=underCrackLeft))
#    
#    xSymFace = mySubInst.faces.getByBoundingBox(cubeX,-1000,-1000,cubeX,1000,1000)
#    mySubModel.XsymmBC(createStepName='General_Sub', localCsys=None, name='Sym_X_Main',
#                region=Region(faces=xSymFace))
#    
#    mySubModel.XsymmBC(createStepName='General_Sub', localCsys=None, name='Sym_X_CrackSide',
#                region=innerFaces)
#            
#    # CRACK FRONT SYMMETRIES
#    originX = cubeX   
#    for i in range(0,(nrInc)+1):
#            radius = crack_depth+(crack_mesh_radius_increment*(i+0.5))
#            xLoc = radius/100
#            yLoc = cubeY-(math.sqrt(math.pow(radius,2)-math.pow(xLoc,2)))
#            bcName = 'Sym_Z_Crack_L_'+str(i)
#            mySubModel.ZsymmBC(createStepName='General_Sub',localCsys=None, name=bcName,
#                    region=Region(faces=mySubInst.faces.findAt(((-xLoc+originX,yLoc,cubeZ),))))    
#        
#    
#    
#    
#    mySubModel.HistoryOutputRequest(contourIntegral='Crack-1', 
#    createStepName='General_Sub', frequency=LAST_INCREMENT, name='Crack', 
#    numberOfContours=nrInc, rebar=EXCLUDE, sectionPoints=DEFAULT)
#                        
#                
#                
#                
#                
#                
#                
#                
#    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
#            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
#            memory=95, memoryUnits=PERCENTAGE, model=sub_name, modelPrint=OFF, 
#            multiprocessingMode=DEFAULT, name=sub_name, nodalOutputPrecision=
#            SINGLE, numCPUs=4, numGPUs=4, numDomains=4, queue=None, resultsFormat=ODB
#            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)          
#        
##    mdb.jobs[sub_name].submit()
##    mdb.jobs[sub_name].waitForCompletion()     
#    
#    
#    
                  
                
      