from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from abaqusConstants import *

import numpy as np
import math

# # # CRACK RATIO FORMULA DEFINED FOR STARTING FROM CIRCULAR

load_array = 100
crack_array_short = np.arange(5,10,1)
crack_array_long = np.arange(10,40,2)
crack_array=np.concatenate([crack_array_short,crack_array_long])
for i in range(0,len(crack_array)):
    # INNIT
    def crackRatioFormula(depth):
        #formula = -1.7737*math.pow(depth,3)+2.7193*math.pow(depth,2)-1.5615*depth + 1.0016 OLD 
        formula = -2.3404*math.pow(depth,3)+3.5332*math.pow(depth,2)-2.2585*depth + 1.1932
        return formula
        
    crack_length = crack_array[i]           
    if crack_length < 10:
        outer_radius=2.5
    else:
        outer_radius = 2.5
    num_circles = 10
    nrLines = 16
    nrDivLin = nrLines/4
    degChanges = 360.0/nrLines*math.pi/180
    radi_increment = outer_radius/num_circles
    crackBuffer=3*outer_radius/num_circles
    
    seedCrackFront = 8
    seedNotchEdge = 25
    global_seed_size = 5
    
    
    circle_end = -outer_radius
    triangle_end=-2*outer_radius
    tri_foot= 1.5*outer_radius

    # Ratio = a/b    
    def ellipseParam(ratio,a):
        sec_coord = a/ratio*math.sqrt(1-math.pow(0.9,2))
        return sec_coord
            
    d = 76.0
    
    centerX = 0
    centerY = d
    centerZ = 0
    


    
    height=76.0




    # Number of mesh thingies
    bend_nr = 16
    bot_leg = 10
    through_nr = 16
    end_under_nr = 3.6 
    
    load = load_array

    if crack_length < 8:
        crackRatio = 1.0
    else:
        crackRatio = crackRatioFormula(crack_length/d)   
    
    print(crack_length,' ',crackRatio)
    
    path = '76mm_'+str(load_array)+'MPa_'+str(int(crack_length))
    mdb.Model(name=path, objectToCopy=
                mdb.models['Source'])    
    myModel = mdb.models[path]
    myPart = myModel.parts['Half link, slave']

    #
    def upperDia(x):
        y = triangle_end-triangle_end/tri_foot*x
        return y
    #
    def lowerDia(x):
        y = -tri_foot+tri_foot/triangle_end*x
        return y

    
    # # # CREATING CRACK SHELL
    crackSweep = myModel.ConstrainedSketch(name='crack_sweep', sheetSize=300)
    crackSweep.EllipseByCenterPerimeter(center=(0,0), axisPoint2=(crack_length,0), axisPoint1=(0,crack_length/crackRatio))
    
    crackShape = myModel.ConstrainedSketch(name='crack_shape', sheetSize=200.0, 
        transform=(-1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -0.0, -1.0, -0.0, crack_length/crackRatio, 0.0, 0.0))
    
    
              
    for i in range(0,nrDivLin+1):
        originY = 0
        degCirc = degChanges*(i)    
        y = -triangle_end/(math.sin(degCirc)+(-triangle_end/tri_foot)*math.cos(degCirc))*math.cos(degCirc)
        x = upperDia(y)
        y_true = y+originY
        y_true_neg = -y+originY
        crackShape.Line(point1=(0,originY),point2=(x,y_true))
        crackShape.Line(point1=(0,originY),point2=(-x,y_true))
        crackShape.Line(point1=(0,originY),point2=(x,y_true_neg))
        crackShape.Line(point1=(0,originY),point2=(-x,y_true_neg))
    #
    #
    for i in range(0,num_circles+1):
        pt = outer_radius-radi_increment*i
        crackShape.CircleByCenterPerimeter(center=(0,originY), point1=(pt,originY))
    #
    #mySketch.Line(point1=(circle_end,0),point2=(triangle_end,0))
    crackShape.Line(point1=(0,tri_foot+originY), point2=(triangle_end,0+originY))
    crackShape.Line(point1=(0,-tri_foot+originY), point2=(triangle_end,0+originY))
    #
    #crackShape.Line(point1=(triangle_end,-200+originY),point2=(triangle_end,originY))
    #crackShape.Line(point1=(triangle_end,originY),point2=(triangle_end,crack_length+originY))
    #
    triangle_end=-triangle_end
    #mySketch.Line(point1=(circle_end,0),point2=(triangle_end,0))
    crackShape.Line(point1=(0,0+tri_foot+originY), point2=(triangle_end,0+originY))
    crackShape.Line(point1=(0,0-tri_foot+originY), point2=(triangle_end,0+originY))
    
    geom = ()
    ids = crackShape.geometry.keys()
    for i in range(0,len(ids)):
        geom = geom + (crackShape.geometry[ids[i]],)
    
    crackShape.rotate(centerPoint=(0,0), angle=90, objectList=geom)
    
    
    myModel.Part(dimensionality=THREE_D, name='Crack', type=
            DEFORMABLE_BODY)
        
    myModel.parts['Crack'].BaseShellSweep(path=crackSweep, sketch=
        crackShape)
        
    del crackShape, crackSweep
    
    
    
    center = (centerX,centerY,centerZ)
    myRoot = myModel.rootAssembly    
    myRoot.Instance(name='Crack', part=myModel.parts['Crack'])
    myRoot.rotate(instanceList=['Crack',], axisPoint=(0,0,0), axisDirection=(0,1,0), angle=90)
    myRoot.rotate(instanceList=['Crack',], axisPoint=(0,0,0), axisDirection=(1,0,0), angle=90)
    myRoot.translate(instanceList=['Crack',], vector=center)
    

    myInst = myRoot.InstanceFromBooleanCut(name='Cracked_Notch', 
                              cuttingInstances=(myRoot.instances['Crack'],),
                              instanceToBeCut=myRoot.instances['Half link, slave-1'], 
                              originalInstances=SUPPRESS)     
                              

    myPart = myModel.parts['Cracked_Notch']
    faceX = myPart.faces.findAt(((0,1,0),))
    x1 = myPart.DatumPlaneByOffset(plane=faceX[0],offset=triangle_end, flip=SIDE2)    
    myPart.PartitionCellByDatumPlane(cells=myPart.cells[0:len(myPart.cells)], 
                                                        datumPlane=myPart.datums[x1.id]) 
    myRoot.makeIndependent(instances=(myInst,))
## # # # SETTING SEEDS ETC. 
#
      
    innerRegion = ()
    innerFaces = ()
    for i in range(0,nrDivLin):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        y = triangle_end/(math.sin(degCirc)+(triangle_end/tri_foot)*math.cos(degCirc))*math.cos(degCirc)
        x = -upperDia(y)
        y_true = y+height-crack_length
        myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((x,y_true,0),))[0],), number=1)
        for j in range(0,num_circles+1):
            radius = outer_radius-radi_increment*j
            line_radius = outer_radius + radi_increment*(0.01-j) 
            yLocCirc = radius*math.cos(degCirc)+(height-crack_length)
            xLocCirc = -radius*math.sin(degCirc)
            yLocLine = (line_radius)*math.cos(degLine)+(height-crack_length)
            xLocLine = -(line_radius)*math.sin(degLine)
            yLocCell = line_radius*math.cos(degCirc)+(height-crack_length)
            xLocCell = -line_radius*math.sin(degCirc)
    #        
            if j == num_circles:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,0),))[0],),number=1)
                myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((xLocCell,yLocCell,0),))[0],), technique=SWEEP)
            else:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,yLocCirc,0),))[0],),number=1)    
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,0),))[0],),number=1)    
                innerRegion = innerRegion + (myInst.cells.findAt(((xLocLine,yLocCirc,0),))[0],)
                innerFaces = innerFaces + (myInst.faces.findAt(((xLocLine,yLocCirc,0),))[0],)
    
    for i in range(0,nrDivLin):
        degCirc = degChanges*(0.5+i)
        degLine = degChanges*(1+i)    
        y = (triangle_end/(math.sin(degCirc)+(triangle_end/tri_foot)*math.cos(degCirc))*math.cos(degCirc))
        x = -upperDia(y)
        y_true = -y+height-crack_length
        myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((x,y_true,0),))[0],), number=1)
        for j in range(0,num_circles+1):
            radius = outer_radius-radi_increment*j
            line_radius = outer_radius + radi_increment*(0.01-j) 
            yLocCirc = -radius*math.cos(degCirc)+(height-crack_length)
            xLocCirc = -radius*math.sin(degCirc)
            yLocLine = -(line_radius)*math.cos(degLine)+(height-crack_length)
            xLocLine = -(line_radius)*math.sin(degLine)
            yLocCell = -line_radius*math.cos(degCirc)+(height-crack_length)
            xLocCell = -line_radius*math.sin(degCirc)
            if j == num_circles:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,0),))[0],),number=1) 
                myRoot.setMeshControls(algorithm=ADVANCING_FRONT,elemShape=HEX_DOMINATED, regions=(myInst.cells.findAt(((xLocCell,yLocCell,0),))[0],), technique=SWEEP)
            else:
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocCirc,yLocCirc,0),))[0],),number=1)    
                myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((xLocLine,yLocLine,0),))[0],),number=1)    
                innerRegion = innerRegion + (myInst.cells.findAt(((xLocLine,yLocCirc,0),))[0],)
                innerFaces = innerFaces + (myInst.faces.findAt(((xLocLine,yLocCirc,0),))[0],)
    
    
    myRoot.setMeshControls(elemShape=HEX_DOMINATED, regions=innerRegion, technique=STRUCTURED)

    # # # SEED FOR CROWN sTOP AREA
    radius = d/2.0
    degRad = 0.5*math.pi/180.0
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((0,radius+radius*math.cos(degRad),radius*math.sin(degRad)),))[0],),number=seedCrackFront/2)
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((0,d-0.01,0),))[0],),number=seedCrackFront/2)
    
#    diff = crack_length-tri_foot
#    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((0,d-diff*0.9,ellipseParam(crackRatio,diff)),))[0],), number=seedCrackFront)
#    
#    diff = crack_length
    #myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((0,d-diff*math.sqrt(1/2.0),diff*math.sqrt(1/2.0)),))[0],), number=seedCrackFront)
    
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((-5.030708,-278.35E-03,0),))[0],), number=5)
    myRoot.seedEdgeByNumber(edges=(myInst.edges.findAt(((-12.85355,-1.849043,0.),))[0],), number=5)   
    myRoot.seedPartInstance(deviationFactor=
                            0.1, minSizeFactor=0.1,
                            regions=(myRoot.instances['Cracked_Notch-1'],), size=global_seed_size) 
#    
 

    # # # DEFINING CRACK
    originX = centerX
    originY = centerY
    originZ = centerZ
    
    outerContour_radius = crack_length-0.5*radi_increment
    zOuterLoc = originZ+0.001
    yOuterLoc = originY-crack_length+0.005       
    
    #
    innerContour_radius = crack_length+0.5*radi_increment    
    zInnerLoc = originZ+0.001
    yInnerLoc = originY-crack_length-0.005
    #
    yCrackLoc = originY-crack_length*0.9
    zCrackLoc = originZ+ellipseParam(crackRatio,crack_length)
        #
            
    crackSurf = myInst.faces.findAt(((originX,yOuterLoc,zOuterLoc),))+myInst.faces.findAt(((originX,yInnerLoc,zInnerLoc),))
    crackEdge = myInst.edges.findAt(((originX,yCrackLoc,zCrackLoc),))
    myRoot.engineeringFeatures.ContourIntegral(
        collapsedElementAtTip=SINGLE_NODE, crackFront=Region(faces=crackSurf,),
        crackNormal=((0.0, 0.0, 
        0.0), (1.0, 0.0, 0.0)),
        crackTip=Region(
        edges=crackEdge,), extensionDirectionMethod=CRACK_NORMAL, 
        midNodePosition=0.3, name='Crack-1', symmetric=ON)    
        
    myRoot.seedEdgeByNumber(edges=crackEdge, number=10)
    myRoot.setElementType(regions=(myInst.cells[0:len(myInst.cells)],), elemTypes=(ElemType(
                                elemCode=C3D20, elemLibrary=STANDARD),))    
    myRoot.generateMesh(regions=(myInst,))   
        
        # # # CRACK FRONT SYMMETRIES BC
    originX = centerX
    for i in range(0,(num_circles)+1):
        radius = crack_length+(radi_increment*(i+0.5))
        zLoc = radius/100
        yLoc = originY-(math.sqrt(math.pow(radius,2)-math.pow(zLoc,2)))
        bcName = 'Sym_X_Crack_'+str(i)
        myModel.XsymmBC(createStepName='Initial', localCsys=None, name=bcName,
                region=Region(faces=myInst.faces.findAt(((originX,yLoc,originZ+zLoc),))))
    
#        
    bcFaceUnder = myInst.faces.findAt(((0.0,0.0+0.01,0.0+0.01),))
    myModel.XsymmBC(createStepName='Initial', localCsys=None, name='bcFaceUnder',
            region=Region(faces=bcFaceUnder))
#  
    bcSymZ = myInst.faces.getByBoundingBox(-1000,-1000,0,1000,1000,0)
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='sym z slave',
            region=innerFaces)    
    myModel.ZsymmBC(createStepName='Initial', localCsys=None, name='sym z2 slave',
            region=Region(faces=bcSymZ))
    bcSymY = myInst.faces.findAt(((-89.3, -151.892048, 0),))
    myModel.YsymmBC(createStepName='Initial', localCsys=None, name='sym y slave',
            region=Region(faces=bcSymY))    
          
    
    myModel.rootAssembly.Surface(name='slave_contact', 
        side1Faces=
        (myInst.faces.findAt(((-11.87456,5.,21.067458),)),myInst.faces.findAt(((-2.508805,5.,18.949893),)),myInst.faces.findAt(((-41.945103,26.715758,38.),)),))
        #
    myModel.interactions['master-slave'].setValues(
        adjustMethod=OVERCLOSED, bondingSet=None, contactTracking=TWO_CONFIG, 
        enforcement=SURFACE_TO_SURFACE, initialClearance=OMIT, slave=
        myRoot.surfaces['slave_contact'], sliding=
        FINITE, thickness=ON, tied=OFF)
    
    
    myModel.HistoryOutputRequest(contourIntegral='Crack-1', 
    createStepName='General', frequency=LAST_INCREMENT, name='Crack', 
    numberOfContours=num_circles, rebar=EXCLUDE, sectionPoints=DEFAULT)      
      
    mdb.Job(atTime=None, contactPrint=OFF, description='Tester', echoPrint=OFF, 
            explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
            memory=95, memoryUnits=PERCENTAGE, model=path, modelPrint=OFF, 
            multiprocessingMode=DEFAULT, name=path, nodalOutputPrecision=
            SINGLE, numCpus=4, numGPUs=0, numDomains=4, queue=None, resultsFormat=ODB
            , scratch='', type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)
    
    mdb.jobs[path].submit()
    mdb.jobs[path].waitForCompletion()
    
